var plugins_bb40132b_638b_4a9f_b028_d3fe47acc8d1 = 
{
    "init": function () {
	// 可以写一些直接执行的代码
	// 在这里写的代码将会在【资源加载前】被执行，此时图片等资源尚未被加载。
	// 请勿在这里对包括bgm，图片等资源进行操作。

	this._afterLoadResources = function () {
		// 本函数将在所有资源加载完毕后，游戏开启前被执行
		// 可以在这个函数里面对资源进行一些操作。
		// 若需要进行切分图片，可以使用 core.splitImage() 函数，或直接在全塔属性-图片切分中操作
	}

	// 可以在任何地方（如afterXXX或自定义脚本事件）调用函数，方法为 core.plugin.xxx();
	// 从V2.6开始，插件中用this.XXX方式定义的函数也会被转发到core中，详见文档-脚本-函数的转发。
},
    "drawLight": function () {

		// 绘制灯光/漆黑层效果。调用方式 core.plugin.drawLight(...)
		// 【参数说明】
		// name：必填，要绘制到的画布名；可以是一个系统画布，或者是个自定义画布；如果不存在则创建
		// color：可选，只能是一个0~1之间的数，为不透明度的值。不填则默认为0.9。
		// lights：可选，一个数组，定义了每个独立的灯光。
		//        其中每一项是三元组 [x,y,r] x和y分别为该灯光的横纵坐标，r为该灯光的半径。
		// lightDec：可选，0到1之间，光从多少百分比才开始衰减（在此范围内保持全亮），不设置默认为0。
		//        比如lightDec为0.5代表，每个灯光部分内圈50%的范围全亮，50%以后才开始快速衰减。
		// 【调用样例】
		// core.plugin.drawLight('curtain'); // 在curtain层绘制全图不透明度0.9，等价于更改画面色调为[0,0,0,0.9]。
		// core.plugin.drawLight('ui', 0.95, [[25,11,46]]); // 在ui层绘制全图不透明度0.95，其中在(25,11)点存在一个半径为46的灯光效果。
		// core.plugin.drawLight('test', 0.2, [[25,11,46,0.1]]); // 创建一个test图层，不透明度0.2，其中在(25,11)点存在一个半径为46的灯光效果，灯光中心不透明度0.1。
		// core.plugin.drawLight('test2', 0.9, [[25,11,46],[105,121,88],[301,221,106]]); // 创建test2图层，且存在三个灯光效果，分别是中心(25,11)半径46，中心(105,121)半径88，中心(301,221)半径106。
		// core.plugin.drawLight('xxx', 0.3, [[25,11,46],[105,121,88,0.2]], 0.4); // 存在两个灯光效果，它们在内圈40%范围内保持全亮，40%后才开始衰减。
		this.drawLight = function (name, color, lights, lightDec) {

			// 清空色调层；也可以修改成其它层比如animate/weather层，或者用自己创建的canvas
			var ctx = core.getContextByName(name);
			if (ctx == null) {
				if (typeof name == 'string')
					ctx = core.createCanvas(name, 0, 0, core.__PIXELS__, core.__PIXELS__, 98);
				else return;
			}

			ctx.mozImageSmoothingEnabled = false;
			ctx.webkitImageSmoothingEnabled = false;
			ctx.msImageSmoothingEnabled = false;
			ctx.imageSmoothingEnabled = false;

			core.clearMap(name);
			// 绘制色调层，默认不透明度
			if (color == null) color = 0.9;
			ctx.fillStyle = "rgba(0,0,0," + color + ")";
			ctx.fillRect(0, 0, ctx.canvas.width, ctx.canvas.height);

			lightDec = core.clamp(lightDec, 0, 1);

			// 绘制每个灯光效果
			ctx.globalCompositeOperation = 'destination-out';
			lights.forEach(function (light) {
				// 坐标，半径，中心不透明度
				var x = light[0],
					y = light[1],
					r = light[2];
				// 计算衰减距离
				var decDistance = parseInt(r * lightDec);
				// 正方形区域的直径和左上角坐标
				var grd = ctx.createRadialGradient(x, y, decDistance, x, y, r);
				grd.addColorStop(0, "rgba(0,0,0,1)");
				grd.addColorStop(1, "rgba(0,0,0,0)");
				ctx.beginPath();
				ctx.fillStyle = grd;
				ctx.arc(x, y, r, 0, 2 * Math.PI);
				ctx.fill();
			});
			ctx.globalCompositeOperation = 'source-over';
			// 可以在任何地方（如afterXXX或自定义脚本事件）调用函数，方法为  core.plugin.xxx();
		}
	},
    "shop": function () {
		// 【全局商店】相关的功能
		// 
		// 打开一个全局商店
		// shopId：要打开的商店id；noRoute：是否不计入录像
		this.openShop = function (shopId, noRoute) {
			var shop = core.status.shops[shopId];
			// Step 1: 检查能否打开此商店
			if (!this.canOpenShop(shopId)) {
				core.drawTip("该商店尚未开启");
				return false;
			}

			// Step 2: （如有必要）记录打开商店的脚本事件
			if (!noRoute) {
				core.status.route.push("shop:" + shopId);
			}

			// Step 3: 检查道具商店 or 公共事件
			if (shop.item) {
				if (core.openItemShop) {
					core.openItemShop(shopId);
				} else {
					core.playSound('操作失败');
					core.insertAction("道具商店插件不存在！请检查是否存在该插件！");
				}
				return;
			}
			if (shop.commonEvent) {
				core.insertCommonEvent(shop.commonEvent, shop.args);
				return;
			}

			_shouldProcessKeyUp = true;

			// Step 4: 执行标准公共商店    
			core.insertAction(this._convertShop(shop));
			return true;
		}

		////// 将一个全局商店转变成可预览的公共事件 //////
		this._convertShop = function (shop) {
			return [
				{ "type": "function", "function": "function() {core.addFlag('@temp@shop', 1);}" },
				{
					"type": "while",
					"condition": "true",
					"data": [
						// 检测能否访问该商店
						{
							"type": "if",
							"condition": "core.isShopVisited('" + shop.id + "')",
							"true": [
								// 可以访问，直接插入执行效果
								{ "type": "function", "function": "function() { core.plugin._convertShop_replaceChoices('" + shop.id + "', false) }" },
							],
							"false": [
								// 不能访问的情况下：检测能否预览
								{
									"type": "if",
									"condition": shop.disablePreview,
									"true": [
										// 不可预览，提示并退出
										{ "type": "playSound", "name": "操作失败" },
										"当前无法访问该商店！",
										{ "type": "break" },
									],
									"false": [
										// 可以预览：将商店全部内容进行替换
										{ "type": "tip", "text": "当前处于预览模式，不可购买" },
										{ "type": "function", "function": "function() { core.plugin._convertShop_replaceChoices('" + shop.id + "', true) }" },
									]
								}
							]
						}
					]
				},
				{ "type": "function", "function": "function() {core.addFlag('@temp@shop', -1);}" }
			];
		}

		this._convertShop_replaceChoices = function (shopId, previewMode) {
			var shop = core.status.shops[shopId];
			var choices = (shop.choices || []).filter(function (choice) {
				if (choice.condition == null || choice.condition == '') return true;
				try { return core.calValue(choice.condition); } catch (e) { return true; }
			}).map(function (choice) {
				var ableToBuy = core.calValue(choice.need);
				return {
					"text": choice.text,
					"icon": choice.icon,
					"color": ableToBuy && !previewMode ? choice.color : [153, 153, 153, 1],
					"action": ableToBuy && !previewMode ? [{ "type": "playSound", "name": "商店" }].concat(choice.action) : [
						{ "type": "playSound", "name": "操作失败" },
						{ "type": "tip", "text": previewMode ? "预览模式下不可购买" : "购买条件不足" }
					]
				};
			}).concat({ "text": "离开", "action": [{ "type": "playSound", "name": "取消" }, { "type": "break" }] });
			core.insertAction({ "type": "choices", "text": shop.text, "choices": choices });
		}

		/// 是否访问过某个快捷商店
		this.isShopVisited = function (id) {
			if (!core.hasFlag("__shops__")) core.setFlag("__shops__", {});
			var shops = core.getFlag("__shops__");
			if (!shops[id]) shops[id] = {};
			return shops[id].visited;
		}

		/// 当前应当显示的快捷商店列表
		this.listShopIds = function () {
			return Object.keys(core.status.shops).filter(function (id) {
				return core.isShopVisited(id) || !core.status.shops[id].mustEnable;
			});
		}

		/// 是否能够打开某个商店
		this.canOpenShop = function (id) {
			if (this.isShopVisited(id)) return true;
			var shop = core.status.shops[id];
			if (shop.item || shop.commonEvent || shop.mustEnable) return false;
			return true;
		}

		/// 启用或禁用某个快捷商店
		this.setShopVisited = function (id, visited) {
			if (!core.hasFlag("__shops__")) core.setFlag("__shops__", {});
			var shops = core.getFlag("__shops__");
			if (!shops[id]) shops[id] = {};
			if (visited) shops[id].visited = true;
			else delete shops[id].visited;
		}

		/// 能否使用快捷商店
		this.canUseQuickShop = function (id) {
			// 如果返回一个字符串，表示不能，字符串为不能使用的提示
			// 返回null代表可以使用

			// 检查当前楼层的canUseQuickShop选项是否为false
			if (core.status.thisMap.canUseQuickShop === false)
				return '当前楼层不能使用快捷商店。';
			return null;
		}

		var _shouldProcessKeyUp = true;

		/// 允许商店X键退出
		core.registerAction('keyUp', 'shops', function (keycode) {
			if (!core.status.lockControl || core.status.event.id != 'action') return false;
			if ((keycode == 13 || keycode == 32) && !_shouldProcessKeyUp) {
				_shouldProcessKeyUp = true;
				return true;
			}

			if (!core.hasFlag("@temp@shop") || core.status.event.data.type != 'choices') return false;
			var data = core.status.event.data.current;
			var choices = data.choices;
			var topIndex = core.actions._getChoicesTopIndex(choices.length);
			if (keycode == 88 || keycode == 27) { // X, ESC
				core.actions._clickAction(core.actions.HSIZE, topIndex + choices.length - 1);
				return true;
			}
			return false;
		}, 60);

		/// 允许长按空格或回车连续执行操作
		core.registerAction('keyDown', 'shops', function (keycode) {
			if (!core.status.lockControl || !core.hasFlag("@temp@shop") || core.status.event.id != 'action') return false;
			if (core.status.event.data.type != 'choices') return false;
			core.status.onShopLongDown = true;
			var data = core.status.event.data.current;
			var choices = data.choices;
			var topIndex = core.actions._getChoicesTopIndex(choices.length);
			if (keycode == 13 || keycode == 32) { // Space, Enter
				core.actions._clickAction(core.actions.HSIZE, topIndex + core.status.event.selection);
				_shouldProcessKeyUp = false;
				return true;
			}
			return false;
		}, 60);

		// 允许长按屏幕连续执行操作
		core.registerAction('longClick', 'shops', function (x, y, px, py) {
			if (!core.status.lockControl || !core.hasFlag("@temp@shop") || core.status.event.id != 'action') return false;
			if (core.status.event.data.type != 'choices') return false;
			var data = core.status.event.data.current;
			var choices = data.choices;
			var topIndex = core.actions._getChoicesTopIndex(choices.length);
			if (x >= core.actions.CHOICES_LEFT && x <= core.actions.CHOICES_RIGHT && y >= topIndex && y < topIndex + choices.length) {
				core.actions._clickAction(x, y);
				return true;
			}
			return false;
		}, 60);
	},
    "removeMap": function () {
		// 高层塔砍层插件，删除后不会存入存档，不可浏览地图也不可飞到。
		// 推荐用法：
		// 对于超高层或分区域塔，当在1区时将2区以后的地图删除；1区结束时恢复2区，进二区时删除1区地图，以此类推
		// 这样可以大幅减少存档空间，以及加快存读档速度

		// 删除楼层
		// core.removeMaps("MT1", "MT300") 删除MT1~MT300之间的全部层
		// core.removeMaps("MT10") 只删除MT10层
		this.removeMaps = function (fromId, toId) {
			toId = toId || fromId;
			var fromIndex = core.floorIds.indexOf(fromId),
				toIndex = core.floorIds.indexOf(toId);
			if (toIndex < 0) toIndex = core.floorIds.length - 1;
			flags.__visited__ = flags.__visited__ || {};
			flags.__removed__ = flags.__removed__ || [];
			flags.__disabled__ = flags.__disabled__ || {};
			flags.__leaveLoc__ = flags.__leaveLoc__ || {};
			for (var i = fromIndex; i <= toIndex; ++i) {
				var floorId = core.floorIds[i];
				if (core.status.maps[floorId].deleted) continue;
				delete flags.__visited__[floorId];
				flags.__removed__.push(floorId);
				delete flags.__disabled__[floorId];
				delete flags.__leaveLoc__[floorId];
				(core.status.autoEvents || []).forEach(function (event) {
					if (event.floorId == floorId && event.currentFloor) {
						core.autoEventExecuting(event.symbol, false);
						core.autoEventExecuted(event.symbol, false);
					}
				});
				core.status.maps[floorId].deleted = true;
				core.status.maps[floorId].canFlyTo = false;
				core.status.maps[floorId].canFlyFrom = false;
				core.status.maps[floorId].cannotViewMap = true;
			}
		}

		// 恢复楼层
		// core.resumeMaps("MT1", "MT300") 恢复MT1~MT300之间的全部层
		// core.resumeMaps("MT10") 只恢复MT10层
		this.resumeMaps = function (fromId, toId) {
			toId = toId || fromId;
			var fromIndex = core.floorIds.indexOf(fromId),
				toIndex = core.floorIds.indexOf(toId);
			if (toIndex < 0) toIndex = core.floorIds.length - 1;
			flags.__removed__ = flags.__removed__ || [];
			for (var i = fromIndex; i <= toIndex; ++i) {
				var floorId = core.floorIds[i];
				if (!core.status.maps[floorId].deleted) continue;
				flags.__removed__ = flags.__removed__.filter(function (f) { return f != floorId; });
				core.status.maps[floorId] = core.loadFloor(floorId);
			}
		}

		// 分区砍层相关
		var inAnyPartition = function (floorId) {
			var inPartition = false;
			(core.floorPartitions || []).forEach(function (floor) {
				var fromIndex = core.floorIds.indexOf(floor[0]);
				var toIndex = core.floorIds.indexOf(floor[1]);
				var index = core.floorIds.indexOf(floorId);
				if (fromIndex < 0 || index < 0) return;
				if (toIndex < 0) toIndex = core.floorIds.length - 1;
				if (index >= fromIndex && index <= toIndex) inPartition = true;
			});
			return inPartition;
		}

		// 分区砍层
		this.autoRemoveMaps = function (floorId) {
			if (main.mode != 'play' || !inAnyPartition(floorId)) return;
			// 根据分区信息自动砍层与恢复
			(core.floorPartitions || []).forEach(function (floor) {
				var fromIndex = core.floorIds.indexOf(floor[0]);
				var toIndex = core.floorIds.indexOf(floor[1]);
				var index = core.floorIds.indexOf(floorId);
				if (fromIndex < 0 || index < 0) return;
				if (toIndex < 0) toIndex = core.floorIds.length - 1;
				if (index >= fromIndex && index <= toIndex) {
					core.resumeMaps(core.floorIds[fromIndex], core.floorIds[toIndex]);
				} else {
					core.removeMaps(core.floorIds[fromIndex], core.floorIds[toIndex]);
				}
			});
		}
	},
    "fiveLayers": function () {
		// 是否启用五图层（增加背景2层和前景2层） 将__enable置为true即会启用；启用后请保存后刷新编辑器
		// 背景层2将会覆盖背景层 被事件层覆盖 前景层2将会覆盖前景层
		// 另外 请注意加入两个新图层 会让大地图的性能降低一些
		// 插件作者：ad
		var __enable = false;
		if (!__enable) return;

		// 创建新图层
		function createCanvas (name, zIndex) {
			if (!name) return;
			var canvas = document.createElement('canvas');
			canvas.id = name;
			canvas.className = 'gameCanvas';
			// 编辑器模式下设置zIndex会导致加入的图层覆盖优先级过高
			if (main.mode != "editor") canvas.style.zIndex = zIndex || 0;
			// 将图层插入进游戏内容
			document.getElementById('gameDraw').appendChild(canvas);
			var ctx = canvas.getContext('2d');
			core.canvas[name] = ctx;
			canvas.width = core.__PIXELS__;
			canvas.height = core.__PIXELS__;
			return canvas;
		}

		var bg2Canvas = createCanvas('bg2', 20);
		var fg2Canvas = createCanvas('fg2', 63);
		// 大地图适配
		core.bigmap.canvas = ["bg2", "fg2", "bg", "event", "event2", "fg", "damage"];
		core.initStatus.bg2maps = {};
		core.initStatus.fg2maps = {};

		if (main.mode == 'editor') {
			/*插入编辑器的图层 不做此步新增图层无法在编辑器显示*/
			// 编辑器图层覆盖优先级 eui > efg > fg(前景层) > event2(48*32图块的事件层) > event(事件层) > bg(背景层)
			// 背景层2(bg2) 插入事件层(event)之前(即bg与event之间)
			document.getElementById('mapEdit').insertBefore(bg2Canvas, document.getElementById('event'));
			// 前景层2(fg2) 插入编辑器前景(efg)之前(即fg之后)
			document.getElementById('mapEdit').insertBefore(fg2Canvas, document.getElementById('ebm'));
			// 原本有三个图层 从4开始添加
			var num = 4;
			// 新增图层存入editor.dom中
			editor.dom.bg2c = core.canvas.bg2.canvas;
			editor.dom.bg2Ctx = core.canvas.bg2;
			editor.dom.fg2c = core.canvas.fg2.canvas;
			editor.dom.fg2Ctx = core.canvas.fg2;
			editor.dom.maps.push('bg2map', 'fg2map');
			editor.dom.canvas.push('bg2', 'fg2');

			// 创建编辑器上的按钮
			var createCanvasBtn = function (name) {
				// 电脑端创建按钮
				var input = document.createElement('input');
				// layerMod4/layerMod5
				var id = 'layerMod' + num++;
				// bg2map/fg2map
				var value = name + 'map';
				input.type = 'radio';
				input.name = 'layerMod';
				input.id = id;
				input.value = value;
				editor.dom[id] = input;
				input.onchange = function () {
					editor.uifunctions.setLayerMod(value);
				}
				return input;
			};

			var createCanvasBtn_mobile = function (name) {
				// 手机端往选择列表中添加子选项
				var input = document.createElement('option');
				var id = 'layerMod' + num++;
				var value = name + 'map';
				input.name = 'layerMod';
				input.value = value;
				editor.dom[id] = input;
				return input;
			};
			if (!editor.isMobile) {
				var input = createCanvasBtn('bg2');
				var input2 = createCanvasBtn('fg2');
				// 获取事件层及其父节点
				var child = document.getElementById('layerMod'),
					parent = child.parentNode;
				// 背景层2插入事件层前
				parent.insertBefore(input, child);
				// 不能直接更改背景层2的innerText 所以创建文本节点
				var txt = document.createTextNode('bg2');
				// 插入事件层前(即新插入的背景层2前)
				parent.insertBefore(txt, child);
				// 向最后插入前景层2(即插入前景层后)
				parent.appendChild(input2);
				var txt2 = document.createTextNode('fg2');
				parent.appendChild(txt2);
				parent.childNodes[2].replaceWith("bg");
				parent.childNodes[6].replaceWith("事件");
				parent.childNodes[8].replaceWith("fg");
			} else {
				var input = createCanvasBtn_mobile('bg2');
				var input2 = createCanvasBtn_mobile('fg2');
				// 手机端因为是选项 所以可以直接改innerText
				input.innerText = '背景层2';
				input2.innerText = '前景层2';
				var parent = document.getElementById('layerMod');
				parent.insertBefore(input, parent.children[1]);
				parent.appendChild(input2);
			}
		}

		var _loadFloor_doNotCopy = core.maps._loadFloor_doNotCopy;
		core.maps._loadFloor_doNotCopy = function () {
			return ["bg2map", "fg2map"].concat(_loadFloor_doNotCopy());
		}
		////// 绘制背景和前景层 //////
		core.maps._drawBg_draw = function (floorId, toDrawCtx, cacheCtx, config) {
			config.ctx = cacheCtx;
			core.maps._drawBg_drawBackground(floorId, config);
			// ------ 调整这两行的顺序来控制是先绘制贴图还是先绘制背景图块；后绘制的覆盖先绘制的。
			core.maps._drawFloorImages(floorId, config.ctx, 'bg', null, null, config.onMap);
			core.maps._drawBgFgMap(floorId, 'bg', config);
			if (config.onMap) {
				core.drawImage(toDrawCtx, cacheCtx.canvas, core.bigmap.v2 ? -32 : 0, core.bigmap.v2 ? -32 : 0);
				core.clearMap('bg2');
				core.clearMap(cacheCtx);
			}
			core.maps._drawBgFgMap(floorId, 'bg2', config);
			if (config.onMap) core.drawImage('bg2', cacheCtx.canvas, core.bigmap.v2 ? -32 : 0, core.bigmap.v2 ? -32 : 0);
			config.ctx = toDrawCtx;
		}
		core.maps._drawFg_draw = function (floorId, toDrawCtx, cacheCtx, config) {
			config.ctx = cacheCtx;
			// ------ 调整这两行的顺序来控制是先绘制贴图还是先绘制前景图块；后绘制的覆盖先绘制的。
			core.maps._drawFloorImages(floorId, config.ctx, 'fg', null, null, config.onMap);
			core.maps._drawBgFgMap(floorId, 'fg', config);
			if (config.onMap) {
				core.drawImage(toDrawCtx, cacheCtx.canvas, core.bigmap.v2 ? -32 : 0, core.bigmap.v2 ? -32 : 0);
				core.clearMap('fg2');
				core.clearMap(cacheCtx);
			}
			core.maps._drawBgFgMap(floorId, 'fg2', config);
			if (config.onMap) core.drawImage('fg2', cacheCtx.canvas, core.bigmap.v2 ? -32 : 0, core.bigmap.v2 ? -32 : 0);
			config.ctx = toDrawCtx;
		}
		////// 移动判定 //////
		core.maps._generateMovableArray_arrays = function (floorId) {
			return {
				bgArray: this.getBgMapArray(floorId),
				fgArray: this.getFgMapArray(floorId),
				eventArray: this.getMapArray(floorId),
				bg2Array: this._getBgFgMapArray('bg2', floorId),
				fg2Array: this._getBgFgMapArray('fg2', floorId)
			};
		}
	},
    "itemShop": function () {
		// 道具商店相关的插件
		// 可在全塔属性-全局商店中使用「道具商店」事件块进行编辑（如果找不到可以在入口方块中找）

		var shopId = null; // 当前商店ID
		var type = 0; // 当前正在选中的类型，0买入1卖出
		var selectItem = 0; // 当前正在选中的道具
		var selectCount = 0; // 当前已经选中的数量
		var page = 0;
		var totalPage = 0;
		var totalMoney = 0;
		var list = [];
		var shopInfo = null; // 商店信息
		var choices = []; // 商店选项
		var use = 'money';
		var useText = '金币';

		var bigFont = core.ui._buildFont(20, false),
			middleFont = core.ui._buildFont(18, false);

		this._drawItemShop = function () {
			// 绘制道具商店

			// Step 1: 背景和固定的几个文字
			core.ui._createUIEvent();
			core.clearMap('uievent');
			core.ui.clearUIEventSelector();
			core.setTextAlign('uievent', 'left');
			core.setTextBaseline('uievent', 'top');
			core.fillRect('uievent', 0, 0, 416, 416, 'black');
			core.drawWindowSkin('winskin.png', 'uievent', 0, 0, 416, 56);
			core.drawWindowSkin('winskin.png', 'uievent', 0, 56, 312, 56);
			core.drawWindowSkin('winskin.png', 'uievent', 0, 112, 312, 304);
			core.drawWindowSkin('winskin.png', 'uievent', 312, 56, 104, 56);
			core.drawWindowSkin('winskin.png', 'uievent', 312, 112, 104, 304);
			core.setFillStyle('uievent', 'white');
			core.setStrokeStyle('uievent', 'white');
			core.fillText("uievent", "购买", 32, 74, 'white', bigFont);
			core.fillText("uievent", "卖出", 132, 74);
			core.fillText("uievent", "离开", 232, 74);
			core.fillText("uievent", "当前" + useText, 324, 66, null, middleFont);
			core.setTextAlign("uievent", "right");
			core.fillText("uievent", core.formatBigNumber(core.status.hero[use]), 405, 89);
			core.setTextAlign("uievent", "left");
			core.ui.drawUIEventSelector(1, "winskin.png", 22 + 100 * type, 66, 60, 33);
			if (selectItem != null) {
				core.setTextAlign('uievent', 'center');
				core.fillText("uievent", type == 0 ? "买入个数" : "卖出个数", 364, 320, null, bigFont);
				core.fillText("uievent", "<   " + selectCount + "   >", 364, 350);
				core.fillText("uievent", "确定", 364, 380);
			}

			// Step 2：获得列表并展示
			list = choices.filter(function (one) {
				if (one.condition != null && one.condition != '') {
					try { if (!core.calValue(one.condition)) return false; } catch (e) { }
				}
				return (type == 0 && one.money != null) || (type == 1 && one.sell != null);
			});
			var per_page = 6;
			totalPage = Math.ceil(list.length / per_page);
			page = Math.floor((selectItem || 0) / per_page) + 1;

			// 绘制分页
			if (totalPage > 1) {
				var half = 156;
				core.setTextAlign('uievent', 'center');
				core.fillText('uievent', page + " / " + totalPage, half, 388, null, middleFont);
				if (page > 1) core.fillText('uievent', '上一页', half - 80, 388);
				if (page < totalPage) core.fillText('uievent', '下一页', half + 80, 388);
			}
			core.setTextAlign('uievent', 'left');

			// 绘制每一项
			var start = (page - 1) * per_page;
			for (var i = 0; i < per_page; ++i) {
				var curr = start + i;
				if (curr >= list.length) break;
				var item = list[curr];
				core.drawIcon('uievent', item.id, 10, 125 + i * 40);
				core.setTextAlign('uievent', 'left');
				core.fillText('uievent', core.material.items[item.id].name, 50, 132 + i * 40, null, bigFont);
				core.setTextAlign('uievent', 'right');
				core.fillText('uievent', (type == 0 ? core.calValue(item.money) : core.calValue(item.sell)) + useText + "/个", 300, 133 + i * 40, null, middleFont);
				core.setTextAlign("uievent", "left");
				if (curr == selectItem) {
					// 绘制描述，文字自动放缩
					var text = core.material.items[item.id].text || "该道具暂无描述";
					try { text = core.replaceText(text); } catch (e) { }
					for (var fontSize = 20; fontSize >= 8; fontSize -= 2) {
						var config = { left: 10, fontSize: fontSize, maxWidth: 403 };
						var height = core.getTextContentHeight(text, config);
						if (height <= 50) {
							config.top = (56 - height) / 2;
							core.drawTextContent("uievent", text, config);
							break;
						}
					}
					core.ui.drawUIEventSelector(2, "winskin.png", 8, 120 + i * 40, 295, 40);
					if (type == 0 && item.number != null) {
						core.fillText("uievent", "存货", 324, 132, null, bigFont);
						core.setTextAlign("uievent", "right");
						core.fillText("uievent", item.number, 406, 132, null, null, 40);
					} else if (type == 1) {
						core.fillText("uievent", "数量", 324, 132, null, bigFont);
						core.setTextAlign("uievent", "right");
						core.fillText("uievent", core.itemCount(item.id), 406, 132, null, null, 40);
					}
					core.setTextAlign("uievent", "left");
					core.fillText("uievent", "预计" + useText, 324, 250);
					core.setTextAlign("uievent", "right");
					totalMoney = selectCount * (type == 0 ? core.calValue(item.money) : core.calValue(item.sell));
					core.fillText("uievent", core.formatBigNumber(totalMoney), 405, 280);

					core.setTextAlign("uievent", "left");
					core.fillText("uievent", type == 0 ? "已购次数" : "已卖次数", 324, 170);
					core.setTextAlign("uievent", "right");
					core.fillText("uievent", (type == 0 ? item.money_count : item.sell_count) || 0, 405, 200);
				}
			}

			core.setTextAlign('uievent', 'left');
			core.setTextBaseline('uievent', 'alphabetic');
		}

		var _add = function (item, delta) {
			if (item == null) return;
			selectCount = core.clamp(
				selectCount + delta, 0,
				Math.min(type == 0 ? Math.floor(core.status.hero[use] / core.calValue(item.money)) : core.itemCount(item.id),
					type == 0 && item.number != null ? item.number : Number.MAX_SAFE_INTEGER)
			);
		}

		var _confirm = function (item) {
			if (item == null || selectCount == 0) return;
			if (type == 0) {
				core.status.hero[use] -= totalMoney;
				core.getItem(item.id, selectCount);
				core.stopSound();
				core.playSound('确定');
				if (item.number != null) item.number -= selectCount;
				item.money_count = (item.money_count || 0) + selectCount;
			} else {
				core.status.hero[use] += totalMoney;
				core.removeItem(item.id, selectCount);
				core.playSound('确定');
				core.drawTip("成功卖出" + selectCount + "个" + core.material.items[item.id].name, item.id);
				if (item.number != null) item.number += selectCount;
				item.sell_count = (item.sell_count || 0) + selectCount;
			}
			selectCount = 0;
		}

		this._performItemShopKeyBoard = function (keycode) {
			var item = list[selectItem] || null;
			// 键盘操作
			switch (keycode) {
				case 38: // up
					if (selectItem == null) break;
					if (selectItem == 0) selectItem = null;
					else selectItem--;
					selectCount = 0;
					break;
				case 37: // left
					if (selectItem == null) {
						if (type > 0) type--;
						break;
					}
					_add(item, -1);
					break;
				case 39: // right
					if (selectItem == null) {
						if (type < 2) type++;
						break;
					}
					_add(item, 1);
					break;
				case 40: // down
					if (selectItem == null) {
						if (list.length > 0) selectItem = 0;
						break;
					}
					if (list.length == 0) break;
					selectItem = Math.min(selectItem + 1, list.length - 1);
					selectCount = 0;
					break;
				case 13:
				case 32: // Enter/Space
					if (selectItem == null) {
						if (type == 2)
							core.insertAction({ "type": "break" });
						else if (list.length > 0)
							selectItem = 0;
						break;
					}
					_confirm(item);
					break;
				case 27: // ESC
					if (selectItem == null) {
						core.insertAction({ "type": "break" });
						break;
					}
					selectItem = null;
					break;
			}
		}

		this._performItemShopClick = function (px, py) {
			var item = list[selectItem] || null;
			// 鼠标操作
			if (px >= 22 && px <= 82 && py >= 71 && py <= 102) {
				// 买
				if (type != 0) {
					type = 0;
					selectItem = null;
					selectCount = 0;
				}
				return;
			}
			if (px >= 122 && px <= 182 && py >= 71 && py <= 102) {
				// 卖
				if (type != 1) {
					type = 1;
					selectItem = null;
					selectCount = 0;
				}
				return;
			}
			if (px >= 222 && px <= 282 && py >= 71 && py <= 102) // 离开
				return core.insertAction({ "type": "break" });
			// < >
			if (px >= 318 && px <= 341 && py >= 348 && py <= 376)
				return _add(item, -1);
			if (px >= 388 && px <= 416 && py >= 348 && py <= 376)
				return _add(item, 1);
			// 确定
			if (px >= 341 && px <= 387 && py >= 380 && py <= 407)
				return _confirm(item);

			// 上一页/下一页
			if (px >= 45 && px <= 105 && py >= 388) {
				if (page > 1) {
					selectItem -= 6;
					selectCount = 0;
				}
				return;
			}
			if (px >= 208 && px <= 268 && py >= 388) {
				if (page < totalPage) {
					selectItem = Math.min(selectItem + 6, list.length - 1);
					selectCount = 0;
				}
				return;
			}

			// 实际区域
			if (px >= 9 && px <= 300 && py >= 120 && py < 360) {
				if (list.length == 0) return;
				var index = parseInt((py - 120) / 40);
				var newItem = 6 * (page - 1) + index;
				if (newItem >= list.length) newItem = list.length - 1;
				if (newItem != selectItem) {
					selectItem = newItem;
					selectCount = 0;
				}
				return;
			}
		}

		this._performItemShopAction = function () {
			if (flags.type == 0) return this._performItemShopKeyBoard(flags.keycode);
			else return this._performItemShopClick(flags.px, flags.py);
		}

		this.openItemShop = function (itemShopId) {
			shopId = itemShopId;
			type = 0;
			page = 0;
			selectItem = null;
			selectCount = 0;
			core.isShopVisited(itemShopId);
			shopInfo = flags.__shops__[shopId];
			if (shopInfo.choices == null) shopInfo.choices = core.clone(core.status.shops[shopId].choices);
			choices = shopInfo.choices;
			use = core.status.shops[shopId].use;
			if (use != 'exp') use = 'money';
			useText = use == 'money' ? '金币' : '经验';

			core.insertAction([{
				"type": "while",
				"condition": "true",
				"data": [
					{ "type": "function", "function": "function () { core.plugin._drawItemShop(); }" },
					{ "type": "wait" },
					{ "type": "function", "function": "function() { core.plugin._performItemShopAction(); }" }
				]
			},
			{
				"type": "function",
				"function": "function () { core.deleteCanvas('uievent'); core.ui.clearUIEventSelector(); }"
			}
			]);
		}

	},
    "enemyLevel": function () {
	// 此插件将提供怪物手册中的怪物境界显示
	// 使用此插件需要先给每个怪物定义境界，方法如下：
	// 点击怪物的【配置表格】，找到“【怪物】相关的表格配置”，然后在【名称】仿照增加境界定义：
	/*
	 "level": {
		  "_leaf": true,
		  "_type": "textarea",
		  "_string": true,
		  "_data": "境界"
	 },
	 */
	// 然后保存刷新，可以看到怪物的属性定义中出现了【境界】。再开启本插件即可。

	// 是否开启本插件，默认禁用；将此改成 true 将启用本插件。
	var __enable = true;
	if (!__enable) return;

	// 这里定义每个境界的显示颜色；可以写'red', '#RRGGBB' 或者[r,g,b,a]四元数组
	var levelToColors = {

	};

	// 复写 _drawBook_drawName
	var originDrawBook = core.ui._drawBook_drawName;
	core.ui._drawBook_drawName = function (index, enemy, top, left, width) {
		// 如果没有境界，则直接调用原始代码绘制
		if (!enemy.level) return originDrawBook.call(core.ui, index, enemy, top, left, width);
		// 存在境界，则额外进行绘制
		core.setTextAlign('ui', 'center');
		if (enemy.specialText.length == 0) {
			core.fillText('ui', enemy.name, left + width / 2,
				top + 27, '#DDDDDD', this._buildFont(17, true));
			core.fillText('ui', enemy.level, left + width / 2,
				top + 51, core.arrayToRGBA(levelToColors[enemy.level] || '#DDDDDD'), this._buildFont(14, true));
		} else {
			core.fillText('ui', enemy.name, left + width / 2,
				top + 20, '#DDDDDD', this._buildFont(17, true), width);
			switch (enemy.specialText.length) {
			case 1:
				core.fillText('ui', enemy.specialText[0], left + width / 2,
					top + 38, core.arrayToRGBA((enemy.specialColor || [])[0] || '#FF6A6A'),
					this._buildFont(14, true), width);
				break;
			case 2:
				// Step 1: 计算字体
				var text = enemy.specialText[0] + "  " + enemy.specialText[1];
				core.setFontForMaxWidth('ui', text, width, this._buildFont(14, true));
				// Step 2: 计算总宽度
				var totalWidth = core.calWidth('ui', text);
				var leftWidth = core.calWidth('ui', enemy.specialText[0]);
				var rightWidth = core.calWidth('ui', enemy.specialText[1]);
				// Step 3: 绘制
				core.fillText('ui', enemy.specialText[0], left + (width + leftWidth - totalWidth) / 2,
					top + 38, core.arrayToRGBA((enemy.specialColor || [])[0] || '#FF6A6A'));
				core.fillText('ui', enemy.specialText[1], left + (width + totalWidth - rightWidth) / 2,
					top + 38, core.arrayToRGBA((enemy.specialColor || [])[1] || '#FF6A6A'));
				break;
			default:
				core.fillText('ui', '多属性...', left + width / 2,
					top + 38, '#FF6A6A', this._buildFont(14, true), width);
			}
			core.fillText('ui', enemy.level, left + width / 2,
				top + 56, core.arrayToRGBA(levelToColors[enemy.level] || '#DDDDDD'), this._buildFont(14, true));
		}
	}

	// 也可以复写其他的属性颜色如怪物攻防等，具体参见下面的例子的注释部分
	core.ui._drawBook_drawRow1 = function (index, enemy, top, left, width, position) {
		// 绘制第一行
		core.setTextAlign('ui', 'left');
		var b13 = this._buildFont(13, true),
			f13 = this._buildFont(13, false);
		var col1 = left,
			col2 = left + width * 9 / 25,
			col3 = left + width * 17 / 25;
		core.fillText('ui', '生命', col1, position, '#DDDDDD', f13);
		core.fillText('ui', core.formatBigNumber(enemy.hp || 0), col1 + 30, position, /*'red' */ null, b13);
		core.fillText('ui', '攻击', col2, position, null, f13);
		core.fillText('ui', core.formatBigNumber(enemy.atk || 0), col2 + 30, position, /* '#FF0000' */ null, b13);
		core.fillText('ui', '防御', col3, position, null, f13);
		core.fillText('ui', core.formatBigNumber(enemy.def || 0), col3 + 30, position, /* [255, 0, 0, 1] */ null, b13);
	}
	//地图显伤在control.js中1230行左右修改

},
    "dynamicHp": function () {
	// 此插件允许人物血量动态进行变化
	// 原作：Fux2（老黄鸡）

	// 是否开启本插件，默认禁用；将此改成 true 将启用本插件。
	var __enable = true;
	if (!__enable) return;

	var speed = 0.05; // 动态血量变化速度，越大越快。

	var _currentHp = null;
	var _lastStatus = null;
	var _check = function () {
		if (_lastStatus != core.status.hero) {
			_lastStatus = core.status.hero;
			_currentHp = core.status.hero.hp;
		}
	}

	core.registerAnimationFrame('dynamicHp', true, function () {
		_check();
		if (core.status.hero.hp != _currentHp) {
			var dis = (_currentHp - core.status.hero.hp) * speed;
			if (Math.abs(dis) < 2) {
				_currentHp = core.status.hero.hp;
			} else {
				_currentHp -= dis;
			}
			core.setStatusBarInnerHTML('hp', _currentHp);
		}
	});
},
    "multiHeros": function () {
		// 多角色插件
		// Step 1: 启用本插件
		// Step 2: 定义每个新的角色各项初始数据（参见下方注释）
		// Step 3: 在游戏中的任何地方都可以调用 `core.changeHero()` 进行切换；也可以 `core.changeHero(1)` 来切换到某个具体的角色上

		// 是否开启本插件，默认禁用；将此改成 true 将启用本插件。
		var __enable = false;
		if (!__enable) return;

		// 在这里定义全部的新角色属性
		// 请注意，在这里定义的内容不会多角色共用，在切换时会进行恢复。
		// 你也可以自行新增或删除，比如不共用金币则可以加上"money"的初始化，不共用道具则可以加上"items"的初始化，
		// 多角色共用hp的话则删除hp，等等。总之，不共用的属性都在这里进行定义就好。
		var hero1 = {
			"floorId": "MT0", // 该角色初始楼层ID；如果共用楼层可以注释此项
			"image": "brave.png", // 角色的行走图名称；此项必填不然会报错
			"name": "1号角色",
			"lv": 1,
			"hp": 10000, // 如果HP共用可注释此项
			"atk": 1000,
			"def": 1000,
			"mdef": 0,
			// "money": 0, // 如果要不共用金币则取消此项注释
			// "exp": 0, // 如果要不共用经验则取消此项注释
			"loc": { "x": 0, "y": 0, "direction": "up" }, // 该角色初始位置；如果共用位置可注释此项
			"items": {
				"tools": {}, // 如果共用消耗道具（含钥匙）则可注释此项
				// "constants": {}, // 如果不共用永久道具（如手册）可取消注释此项
				"equips": {}, // 如果共用在背包的装备可注释此项
			},
			"equipment": [], // 如果共用装备可注释此项；此项和上面的「共用在背包的装备」需要拥有相同状态，不然可能出现问题
		};
		// 也可以类似新增其他角色
		// 新增的角色，各项属性共用与不共用的选择必须和上面完全相同，否则可能出现问题。
		// var hero2 = { ...

		var heroCount = 2; // 包含默认角色在内总共多少个角色，该值需手动修改。

		this.initHeros = function () {
			core.setFlag("hero1", core.clone(hero1)); // 将属性值存到变量中
			// core.setFlag("hero2", core.clone(hero2)); // 更多的角色也存入变量中；每个定义的角色都需要新增一行

			// 检测是否存在装备
			if (hero1.equipment) {
				if (!hero1.items || !hero1.items.equips) {
					alert('多角色插件的equipment和道具中的equips必须拥有相同状态！');
				}
				// 存99号套装为全空
				var saveEquips = core.getFlag("saveEquips", []);
				saveEquips[99] = [];
				core.setFlag("saveEquips", saveEquips);
			} else {
				if (hero1.items && hero1.items.equips) {
					alert('多角色插件的equipment和道具中的equips必须拥有相同状态！');
				}
			}
		}

		// 在游戏开始注入initHeros
		var _startGame_setHard = core.events._startGame_setHard;
		core.events._startGame_setHard = function () {
			_startGame_setHard.call(core.events);
			core.initHeros();
		}

		// 切换角色
		// 可以使用 core.changeHero() 来切换到下一个角色
		// 也可以 core.changeHero(1) 来切换到某个角色（默认角色为0）
		this.changeHero = function (toHeroId) {
			var currHeroId = core.getFlag("heroId", 0); // 获得当前角色ID
			if (toHeroId == null) {
				toHeroId = (currHeroId + 1) % heroCount;
			}
			if (currHeroId == toHeroId) return;

			var saveList = Object.keys(hero1);

			// 保存当前内容
			var toSave = {};
			// 暂时干掉 drawTip 和 音效，避免切装时的提示
			var _drawTip = core.ui.drawTip;
			core.ui.drawTip = function () { };
			var _playSound = core.control.playSound;
			core.control.playSound = function () { }
			// 记录当前录像，因为可能存在换装问题
			core.clearRouteFolding();
			var routeLength = core.status.route.length;
			// 优先判定装备
			if (hero1.equipment) {
				core.items.quickSaveEquip(100 + currHeroId);
				core.items.quickLoadEquip(99);
			}

			saveList.forEach(function (name) {
				if (name == 'floorId') toSave[name] = core.status.floorId; // 楼层单独设置
				else if (name == 'items') {
					toSave.items = core.clone(core.status.hero.items);
					Object.keys(toSave.items).forEach(function (one) {
						if (!hero1.items[one]) delete toSave.items[one];
					});
				} else toSave[name] = core.clone(core.status.hero[name]); // 使用core.clone()来创建新对象
			});

			core.setFlag("hero" + currHeroId, toSave); // 将当前角色信息进行保存
			var data = core.getFlag("hero" + toHeroId); // 获得要切换的角色保存内容

			// 设置角色的属性值
			saveList.forEach(function (name) {
				if (name == "floorId");
				else if (name == "items") {
					Object.keys(core.status.hero.items).forEach(function (one) {
						if (data.items[one]) core.status.hero.items[one] = core.clone(data.items[one]);
					});
				} else {
					core.status.hero[name] = core.clone(data[name]);
				}
			});
			// 最后装上装备
			if (hero1.equipment) {
				core.items.quickLoadEquip(100 + toHeroId);
			}

			core.ui.drawTip = _drawTip;
			core.control.playSound = _playSound;
			core.status.route = core.status.route.slice(0, routeLength);
			core.control._bindRoutePush();

			// 插入事件：改变角色行走图并进行楼层切换
			var toFloorId = data.floorId || core.status.floorId;
			var toLoc = data.loc || core.status.hero.loc;
			core.insertAction([
				{ "type": "setHeroIcon", "name": data.image || "hero.png" }, // 改变行走图
				// 同层则用changePos，不同层则用changeFloor；这是为了避免共用楼层造成触发eachArrive
				toFloorId != core.status.floorId ? {
					"type": "changeFloor",
					"floorId": toFloorId,
					"loc": [toLoc.x, toLoc.y],
					"direction": toLoc.direction,
					"time": 0 // 可以在这里设置切换时间
				} : { "type": "changePos", "loc": [toLoc.x, toLoc.y], "direction": toLoc.direction }
				// 你还可以在这里执行其他事件，比如增加或取消跟随效果
			]);
			core.setFlag("heroId", toHeroId); // 保存切换到的角色ID
		}
	},
    "itemCategory": function () {
		// 物品分类插件。此插件允许你对消耗道具和永久道具进行分类，比如标记「宝物类」「剧情道具」「药品」等等。
		// 使用方法：
		// 1. 启用本插件
		// 2. 在下方数组中定义全部的物品分类类型
		// 3. 点击道具的【配置表格】，找到“【道具】相关的表格配置”，然后在【道具描述】之后仿照增加道具的分类：
		/*
		 "category": {
			  "_leaf": true,
			  "_type": "textarea",
			  "_string": true,
			  "_data": "道具分类"
		 },
		 */
		// （你也可以选择使用下拉框的方式定义每个道具的分类，写法参见上面的cls）
		// 然后刷新编辑器，就可以对每个物品进行分类了

		// 是否开启本插件，默认禁用；将此改成 true 将启用本插件。
		var __enable = false;
		if (!__enable) return;

		// 在这里定义所有的道具分类类型，一行一个
		var categories = [
			"宝物类",
			"辅助类",
			"技能类",
			"剧情道具",
			"增益道具",
		];
		// 当前选中的道具类别
		var currentCategory = null;

		// 重写 core.ui._drawToolbox 以绘制分类类别
		var _drawToolbox = core.ui._drawToolbox;
		core.ui._drawToolbox = function (index) {
			_drawToolbox.call(this, index);
			core.setTextAlign('ui', 'left');
			core.fillText('ui', '类别[E]：' + (currentCategory || "全部"), 15, this.PIXEL - 13);
		}

		// 获得所有应该在道具栏显示的某个类型道具
		core.ui.getToolboxItems = function (cls) {
			// 检查类别
			return Object.keys(core.status.hero.items[cls])
				.filter(function (id) {
					return !core.material.items[id].hideInToolbox &&
						(currentCategory == null || core.material.items[id].category == currentCategory);
				}).sort();
		}

		// 注入道具栏的点击事件（点击类别）
		var _clickToolbox = core.actions._clickToolbox;
		core.actions._clickToolbox = function (x, y) {
			if (x >= 0 && x <= this.HSIZE - 4 && y == this.LAST) {
				drawToolboxCategory();
				return;
			}
			return _clickToolbox.call(core.actions, x, y);
		}

		// 注入道具栏的按键事件（E键）
		var _keyUpToolbox = core.actions._keyUpToolbox;
		core.actions._keyUpToolbox = function (keyCode) {
			if (keyCode == 69) {
				// 按E键则打开分类类别选择
				drawToolboxCategory();
				return;
			}
			return _keyUpToolbox.call(core.actions, keyCode);
		}

		// ------ 以下为选择道具分类的相关代码 ------ //

		// 关闭窗口时清除分类选择项
		var _closePanel = core.ui.closePanel;
		core.ui.closePanel = function () {
			currentCategory = null;
			_closePanel.call(core.ui);
		}

		// 弹出菜单以选择具体哪个分类
		// 直接使用 core.drawChoices 进行绘制
		var drawToolboxCategory = function () {
			if (core.status.event.id != 'toolbox') return;
			var selection = categories.indexOf(currentCategory) + 1;
			core.ui.closePanel();
			core.status.event.id = 'toolbox-category';
			core.status.event.selection = selection;
			core.lockControl();
			// 给第一项插入「全部」
			core.drawChoices('请选择道具类别', ["全部"].concat(categories));
		}

		// 选择某一项
		var _selectCategory = function (index) {
			core.ui.closePanel();
			if (index <= 0 || index > categories.length) currentCategory = null;
			else currentCategory = categories[index - 1];
			core.openToolbox();
		}

		var _clickToolBoxCategory = function (x, y) {
			if (!core.status.lockControl || core.status.event.id != 'toolbox-category') return false;

			if (x < core.actions.CHOICES_LEFT || x > core.actions.CHOICES_RIGHT) return false;
			var choices = core.status.event.ui.choices;
			var topIndex = core.actions._getChoicesTopIndex(choices.length);
			if (y >= topIndex && y < topIndex + choices.length) {
				_selectCategory(y - topIndex);
			}
			return true;
		}

		// 注入点击事件
		core.registerAction('onclick', 'toolbox-category', _clickToolBoxCategory, 100);

		// 注入光标跟随事件
		core.registerAction('onmove', 'toolbox-category', function (x, y) {
			if (!core.status.lockControl || core.status.event.id != 'toolbox-category') return false;
			core.actions._onMoveChoices(x, y);
			return true;
		}, 100);

		// 注入键盘光标事件
		core.registerAction('keyDown', 'toolbox-category', function (keyCode) {
			if (!core.status.lockControl || core.status.event.id != 'toolbox-category') return false;
			core.actions._keyDownChoices(keyCode);
			return true;
		}, 100);

		// 注入键盘按键事件
		core.registerAction('keyUp', 'toolbox-category', function (keyCode) {
			if (!core.status.lockControl || core.status.event.id != 'toolbox-category') return false;
			core.actions._selectChoices(core.status.event.ui.choices.length, keyCode, _clickToolBoxCategory);
			return true;
		}, 100);

	},
    "heroFourFrames": function () {
		// 样板的勇士/跟随者移动时只使用2、4两帧，观感较差。本插件可以将四帧全用上。

		// 是否启用本插件
		var __enable = false;
		if (!__enable) return;

		["up", "down", "left", "right"].forEach(function (one) {
			// 指定中间帧动画
			core.material.icons.hero[one].midFoot = 2;
		});

		var heroMoving = function (timestamp) {
			if (core.status.heroMoving <= 0) return;
			if (timestamp - core.animateFrame.moveTime > core.values.moveSpeed) {
				core.animateFrame.leftLeg++;
				core.animateFrame.moveTime = timestamp;
			}
			core.drawHero(['stop', 'leftFoot', 'midFoot', 'rightFoot'][core.animateFrame.leftLeg % 4], 4 * core.status.heroMoving);
		}
		core.registerAnimationFrame('heroMoving', true, heroMoving);

		core.events._eventMoveHero_moving = function (step, moveSteps) {
			var curr = moveSteps[0];
			var direction = curr[0], x = core.getHeroLoc('x'), y = core.getHeroLoc('y');
			// ------ 前进/后退
			var o = direction == 'backward' ? -1 : 1;
			if (direction == 'forward' || direction == 'backward') direction = core.getHeroLoc('direction');
			var faceDirection = direction;
			if (direction == 'leftup' || direction == 'leftdown') faceDirection = 'left';
			if (direction == 'rightup' || direction == 'rightdown') faceDirection = 'right';
			core.setHeroLoc('direction', direction);
			if (curr[1] <= 0) {
				core.setHeroLoc('direction', faceDirection);
				moveSteps.shift();
				return true;
			}
			if (step <= 4) core.drawHero('stop', 4 * o * step);
			else if (step <= 8) core.drawHero('leftFoot', 4 * o * step);
			else if (step <= 12) core.drawHero('midFoot', 4 * o * (step - 8));
			else if (step <= 16) core.drawHero('rightFoot', 4 * o * (step - 8)); // if (step == 8) {
			if (step == 8 || step == 16) {
				core.setHeroLoc('x', x + o * core.utils.scan2[direction].x, true);
				core.setHeroLoc('y', y + o * core.utils.scan2[direction].y, true);
				core.updateFollowers();
				curr[1]--;
				if (curr[1] <= 0) moveSteps.shift();
				core.setHeroLoc('direction', faceDirection);
				return step == 16;
			}
			return false;
		}
	},
    "startCanvas": function () {
		// 使用本插件可以将自绘的标题界面居中。仅在【标题开启事件化】后才有效。
		// 由于一些技术性的原因，标题界面事件化无法应用到覆盖状态栏的整个界面。
		// 这是一个较为妥协的插件，会在自绘标题界面时隐藏状态栏、工具栏和边框，并将画布进行居中。
		// 本插件仅在全塔属性的 "startCanvas" 生效；进入 "startText" 时将会离开居中状态，回归正常界面。

		// 是否开启本插件，默认禁用；将此改成 true 将启用本插件。
		var __enable = false;
		if (!__enable) return;

		// 检查【标题开启事件化】是否开启
		if (!core.flags.startUsingCanvas || main.mode != 'play') return;

		var _isTitleCanvasEnabled = false;
		var _getClickLoc = core.actions._getClickLoc;
		this._setTitleCanvas = function () {
			if (_isTitleCanvasEnabled) return;
			_isTitleCanvasEnabled = true;

			// 禁用窗口resize
			window.onresize = function () { };
			core.resize = function () { }

			// 隐藏状态栏
			core.dom.statusBar.style.display = 'none';
			core.dom.statusCanvas.style.display = 'none';
			core.dom.toolBar.style.display = 'none';
			// 居中画布
			if (core.domStyle.isVertical) {
				core.dom.gameDraw.style.top =
					(parseInt(core.dom.gameGroup.style.height) - parseInt(core.dom.gameDraw.style.height)) / 2 + "px";
			} else {
				core.dom.gameDraw.style.right =
					(parseInt(core.dom.gameGroup.style.width) - parseInt(core.dom.gameDraw.style.width)) / 2 + "px";
			}
			core.dom.gameDraw.style.border = '3px transparent solid';
			core.actions._getClickLoc = function (x, y) {
				var left = core.dom.gameGroup.offsetLeft + core.dom.gameDraw.offsetLeft + 3;
				var top = core.dom.gameGroup.offsetTop + core.dom.gameDraw.offsetTop + 3;
				var loc = { 'x': Math.max(x - left, 0), 'y': Math.max(y - top, 0), 'size': 32 * core.domStyle.scale };
				return loc;
			}
		}

		this._resetTitleCanvas = function () {
			if (!_isTitleCanvasEnabled) return;
			_isTitleCanvasEnabled = false;
			window.onresize = function () { try { main.core.resize(); } catch (ee) { console.error(ee) } }
			core.resize = function () { return core.control.resize(); }
			core.resize();
			core.actions._getClickLoc = _getClickLoc;
		}

		// 复写“开始游戏”
		core.events._startGame_start = function (hard, seed, route, callback) {
			console.log('开始游戏');
			core.resetGame(core.firstData.hero, hard, null, core.cloneArray(core.initStatus.maps));
			core.setHeroLoc('x', -1);
			core.setHeroLoc('y', -1);

			if (seed != null) {
				core.setFlag('__seed__', seed);
				core.setFlag('__rand__', seed);
			} else core.utils.__init_seed();

			core.clearStatusBar();
			core.plugin._setTitleCanvas();

			var todo = [];
			core.hideStatusBar();
			core.push(todo, core.firstData.startCanvas);
			core.push(todo, { "type": "function", "function": "function() { core.plugin._resetTitleCanvas(); core.events._startGame_setHard(); }" })
			core.push(todo, core.firstData.startText);
			this.insertAction(todo, null, null, function () {
				core.events._startGame_afterStart(callback);
			});

			if (route != null) core.startReplay(route);
		}

		var _loadData = core.control.loadData;
		core.control.loadData = function (data, callback) {
			core.plugin._resetTitleCanvas();
			_loadData.call(core.control, data, callback);
		}
	},
    "核心机制": function () {
	this.getSwordOrShield = function (life, levelOfMonster) {

		var theNumber = life.toString();
		var gettimes = parseInt(levelOfMonster.replace("Lv", ""));
		var IgetSword = 0;
		var canGetSword = true;
		var IgetShield = 0;
		var canGetShield = true;
		var NumberArray = [];

		for (var i = (theNumber.length - 1); i >= 0; i--) {
			NumberArray.push(theNumber[i]);
		}

		function judgeTheFirst(arr) {
			if (arr[0] === "0") {
				arr.shift();
			} else if (arr[0] === "1") {
				arr.shift();
				if (canGetSword === true) {
					canGetShield = false;
					IgetSword += 1;
				}
			} else if (arr[0] === "9") {
				arr.shift();
				if (canGetShield === true) {
					canGetSword = false;
					IgetShield += 1;
				}
			} else {
				arr.shift();
				canGetShield = false;
				canGetSword = false;
			}
		}

		while (NumberArray.length > 0) {
			judgeTheFirst(NumberArray)
		}

		if (flags.followed) {
			if (IgetSword > 0) { IgetSword += 1 }
			if (IgetShield > 0) { IgetShield += 1 }
		}

		if (IgetSword > 0) {
			hero.atk += Math.pow(2, IgetSword) * gettimes / 2;
			core.ui.drawTip("获得" + IgetSword + "阶Lv" + gettimes + "剑，攻击增加" + (Math.pow(2, IgetSword) * gettimes) / 2)
			if (flags.followed && (!flags.followed1)) { core.insertCommonEvent("followed1") }
		}

		if (IgetShield > 0) {
			hero.def += Math.pow(2, IgetShield) * gettimes / 2;
			core.ui.drawTip("获得" + IgetShield + "阶Lv" + gettimes + "盾，防御增加" + (Math.pow(2, IgetShield) * gettimes) / 2)
			if (flags.followed && (!flags.followed1)) { core.insertCommonEvent("followed1") }
		}
	}
},
    "自动拾取": function () {
	var enable = true;
	if (!enable) return;
	// 
	// var noUpdate = false;
	////// 更新状态栏 ////// 不建议状态栏刷新后触发 容易导致录像不一致的问题
	//control.prototype.updateStatusBar = function (doNotCheckAutoEvents) {
	//	if (!core.isPlaying()) return;
	//	if (noUpdate) return;
	//	noUpdate = true;
	//	core.autoGetItem();
	//	noUpdate = false;
	//	this.controldata.updateStatusBar();
	//	if (!doNotCheckAutoEvents) core.checkAutoEvents();
	//	this._updateStatusBar_setToolboxIcon();
	//	core.clearRouteFolding();
	//}

	////// 每移动一格后执行的事件 //////
	//control.prototype.moveOneStep = function (callback) {
	//	core.autoGetItem();
	//	return this.controldata.moveOneStep(callback);
	//}

	function bfsFlood(sx, sy, blockfn) {
		var canMoveArray = core.generateMovableArray();
		var blocksObj = core.getMapBlocksObj();
		var bgMap = core.getBgMapArray();

		var visited = [],
			queue = [];
		visited[sx + "," + sy] = 0;
		queue.push(sx + "," + sy);

		while (queue.length > 0) {
			var now = queue.shift().split(","),
				x = ~~now[0],
				y = ~~now[1];
			for (var direction in core.utils.scan) {
				if (!core.inArray(canMoveArray[x][y], direction)) continue;
				var nx = x + core.utils.scan[direction].x,
					ny = y + core.utils.scan[direction].y,
					nindex = nx + "," + ny;
				if (visited[nindex]) continue;
				if (core.onSki(bgMap[ny][nx])) continue;
				if (blockfn && !blockfn(blocksObj, nx, ny)) continue;
				visited[nindex] = visited[now] + 1;
				queue.push(nindex);
			}
		}
	}

	function attractAnimate() {
		var name = 'attractAnimate';
		var isPlaying = false;
		this.nodes = [];

		this.add = function (id, x, y, callback) {
			this.nodes.push({ id: id, x: x, y: y, callback: callback });
		}
		this.start = function () {
			if (isPlaying) return;
			isPlaying = true;
			core.registerAnimationFrame(name, true, this.update);
			this.ctx = core.createCanvas(name, 0, 0, core.__PIXELS__, core.__PIXELS__, 120);
		}
		this.remove = function () {
			core.unregisterAnimationFrame(name);
			core.deleteCanvas(name);
			isPlaying = false;
		}
		this.clear = function () {
			this.nodes = [];
			this.remove();
		}
		var lastTime = -1;
		var self = this;
		this.update = function (timeStamp) {
			if (lastTime < 0) lastTime = timeStamp;
			if (timeStamp - lastTime < 20) return;
			lastTime = timeStamp;
			core.clearMap(name);
			var cx = core.status.heroCenter.px - 16,
				cy = core.status.heroCenter.py - 16;
			var thr = 1; //缓动比例倒数 越大移动越慢
			self.nodes.forEach(function (n) {
				var dx = cx - n.x,
					dy = cy - n.y;
				if (Math.abs(dx) <= thr && Math.abs(dy) <= thr) {
					n.dead = true;
				} else {
					n.x += ~~(dx / thr);
					n.y += ~~(dy / thr);
				}
				core.drawIcon(name, n.id, n.x, n.y, 32, 32);
			});
			self.nodes = self.nodes.filter(function (n) {
				if (n.dead && n.callback) {
					n.callback();
				}
				return !n.dead;
			});
			if (self.nodes.length == 0)
				self.remove();
		}
	}


	var animateHwnd = new attractAnimate();

	this.stopAttractAnimate = function () {
		animateHwnd.clear();
	}

	this.autoGetItem = function () {
		var canGetItems = {};
		if (!core.status.floorId || !core.status.checkBlock.damage || core.status.event.id == 'action' || core.status.lockControl) return;

		bfsFlood(core.getHeroLoc('x'), core.getHeroLoc('y'), function (blockMap, x, y) {
			var idx = x + ',' + y;
			if (idx in canGetItems) return false;
			var blk = blockMap[idx];
			if (blk && !blk.disable && blk.event.cls == 'items' && !core.isMapBlockDisabled(core.status.floorId, blk.x, blk.y) && blk.event.trigger == 'getItem') {
				canGetItems[idx] = { x: x, y: y, id: blk.event.id };
				return !core.status.checkBlock.damage[idx] && !core.status.checkBlock.ambush[idx];
			}
			return core.maps._canMoveDirectly_checkNextPoint(blockMap, x, y);
		});
		for (var k in canGetItems) {
			var x = canGetItems[k].x,
				y = canGetItems[k].y,
				id = canGetItems[k].id;
			core.trigger(x, y);
			animateHwnd.add(id, x * 32, y * 32);
		}
		animateHwnd.start();
	}
},
    "平面地图楼传": function () {
	// 在此增加新插件
	/* 第一步：复制到插件
	 * 第二步：将flag:usePlatFly改为true可将带地图的楼传启用
	 * 第三步：将flag:__useMinimap__设为true可开启小地图，该操作需切换楼层
	 * 修改小地图的缩放可以修改flag:userScale，默认为1
	 * 注意：楼层转换必须使用楼层坐标，而不是前一楼、后一楼
	 * 关于操作：上楼和下楼操作为PgUp和PgDn，后退10层和前进10层操作为,（<）和.（>）
	 * 注意：请尽量保证地图传送的地方物理位置正好对准（即把地图拼接上箭头位置恰好对齐）否则连线的位置可能不准
	 * 分区说明：用普通事件的楼层转换（红点）代替楼层转换（绿点）即可
	 * 说明：紫色表示目前可以到达却没有到达过的地图（所以这是一个具有探索性的地图插件）
	 */
	// 录像验证直接干掉这个插件
	if (main.replayChecking) return;
	// *** --- 以下数据为用户可修改数据 修改后不影响该插件的基本功能 --- *** //
	// 检测楼层转换的图块id
	var leftPortal = "leftPortal", // 左
		rightPortal = "rightPortal", // 右
		upPortal = "upPortal", // 上
		downPortal = "downPortal", // 下
		upFloor = "upFloor", // 上楼
		downFloor = "downFloor"; // 下楼
	// 一些常用默认值
	var defaultScale = 1, // 默认缩放比率
		defaultLoop = 10, // 绘制地图时的循环检测地图路线次数，loop为5说明最远的地图可以用6步到达
		defaultOpacity = 0.6, // 默认不透明度
		defaultMinorAlpha = defaultOpacity / 2; // 3D绘图时，不与当前层处于同一高度层的默认初始不透明度，不透明度会随着层数的增加或减少而减少
	// *** --- 用户修改区 END --- *** //
	//  其余可自定义内容均用 //***--- 包裹着

	/* -----------------------------------下面是一些调用案例
	 * 
	 * 如果想在状态栏显示可以在状态栏自绘里这么写：
	 core.drawFlyMap(ctx, 你想要的左上角横坐标, 你想要的左上角纵坐标, 你想要的宽度, 你想要的高度, core.status.floorId, {fromUser: true, opacity: 1, loop: 3, use3D:false});
	 * 
	 * -----------------------------------以下为高深区域，如果不必要可以不看
	 * 
	 * 主要函数说明（这几个函数异常强大，善用可以有弄一些有意思的东西，既然作者都说强大了，那就是特别强大）
	 * core.drawFlyMap(ctx : string, x?: number, y?: number, width?:number, height?:number, floorId?:string, config?:any): void
	 * 参数说明：
	 * ctx:画布，如果不存在则会新建x,y,width,height参数的画布
	 * x,y,width,height:画布不存在时创建该数据的画布，如果存在，那么在画布的x,y位置绘制宽度为width，高度为height的地图
	 * floorId:以该楼层为中心绘制
	 * config:配制参数，包括以下内容（该参数为对象）
	 * 	 fromUser:是否循环绘制（为false时只绘制一层，否则绘制5层）
	 * 	 oriFloor:这个不用管就好，具体原理比较复杂，不填或者和floorId一样就行
	 * 	 scale:缩放比率，这个是绝对比率，与画布大小无关，默认为1
	 * 	 interval:地图间的间距，默认为width / 24
	 * 	 deltaH:3D地图的纵向高度差，默认为绘制高度（height）/ 4
	 * 	 noErase:是否不清空画布再绘制，默认为false
	 * 	 fromMini:是否是在小地图上绘制时调用的（无自动缩放和自动定位）
	 * 	 loop:绘制层数，fromUser为true时才有用，默认为5
	 * 	 opacity:不透明度，默认为0.6
	 * 	 minorAlpha:3D地图的不与当前层在同一高度的楼层的初始不透明度，默认为opacity/2
	 * 	 layer:平面模式下绘制的楼层高度，当前所在层为0，上楼则+1，下楼则-1，默认为0，只有当use3D为false且可以绘制3D地图时有效
	 * 	 use3D:是否启用3D绘图，前提是所有地图不都在同一高度，默认为false
	 * 	 reLeft:3D重新定位画布的左位置，默认为-240
	 * 	 reTop:3D重新定位画布的上位置，默认为-240
	 * 	 clearCache:是否清除缓存重算，默认为false
	 * 	 map:这个是最强大的功能了，可以将自定义地图插入，以绘制自定义地图
	 * 	 自定义地图格式：{"left_0_7":"MT1", "right_2_7":"MT2"}以此类推也可以"left_0_7,right_3_7,up_1_0"这种套娃，使用的前提是指定位置得有相应方向的箭头，不然会出错，注意楼层id是到往的楼层的id，如果想绘制3D地图，可以这么写：{"top_1_3,left_0_7": "1_MT1"}，前面的1_必须写，上楼为top，下楼为bottom
	 * 	 默认取自floorId地图
	 * 如有疑问可在造塔群@古祠
	 * 
	 * core.getFlyMap(floorId?:string, fromUser?:boolean, oriFloor?:string, loop?:number):Object
	 * 参数说明:floorId:以该楼层为中心绘制
	 * fromUser:是否循环绘制（为false时只绘制一层，否则绘制5层）
	 * oriFloor:这个不用管就好，具体原理比较复杂，填null或者和floorId一样就行
	 * loop:绘制层数，fromUser为true时才有用，默认为5
	 */

	////// 绘制楼层传送器 //////
	var originDrawFly = core.ui.drawFly;
	ui.prototype.drawFly = function (page) {
		if (!flags.usePlatFly || core.isReplaying()) return originDrawFly.call(core.ui, page);
		core.status.event.data = page;
		var floorId = core.floorIds[page];
		core.clearMap('ui');
		core.setAlpha('ui', 0.85);
		core.fillRect('ui', 0, 0, this.PIXEL, this.PIXEL, '#000000');
		core.setAlpha('ui', 1);
		var size = this.PIXEL;
		//***--- 楼传绘制 可根据自己的需求更改 更改之后注意更改点击操作的函数
		// 背景
		core.drawThumbnail(floorId, null, { ctx: 'ui', x: 0, y: 0, size: size, damage: true, fromFly: true });
		core.fillRect("ui", 0, 65, 32, size - 130, [0, 0, 0, 0.7]);
		core.fillRect("ui", size, 65, -32, size - 130, [0, 0, 0, 0.7]);
		core.fillRect("ui", 65, 0, size / 2 - 114, 32, [0, 0, 0, 0.7]);
		core.fillRect("ui", 65, size, size / 2 - 114, -32, [0, 0, 0, 0.7]);
		core.fillRect("ui", size / 2 + 49, 0, size / 2 - 114, 32, [0, 0, 0, 0.7]);
		core.fillRect("ui", size / 2 + 49, size, size / 2 - 114, -32, [0, 0, 0, 0.7]);
		core.fillRect("ui", size / 2 - 47, 0, 94, 32, [0, 0, 0, 0.7]);
		core.fillRect("ui", size / 2 - 47, size, 94, -32, [0, 0, 0, 0.7]);
		core.fillRect("ui", 0, 0, 63, 32, [0, 0, 0, 0.7]);
		core.fillRect("ui", size, 0, -63, 32, [0, 0, 0, 0.7]);
		core.fillRect("ui", 0, 32, 32, 31, [0, 0, 0, 0.7]);
		core.fillRect("ui", 0, size - 32, 32, -31, [0, 0, 0, 0.7]);
		core.fillRect("ui", 0, size, 63, -32, [0, 0, 0, 0.7]);
		core.fillRect("ui", size, size, -63, -32, [0, 0, 0, 0.7]);
		core.fillRect("ui", size, size - 32, -32, -31, [0, 0, 0, 0.7]);
		core.fillRect("ui", size, 32, -32, 31, [0, 0, 0, 0.7]);
		core.setTextAlign("ui", "center");
		// 文字
		if (core.getFloorByDirection("left", floorId))
			core.fillText("ui", "←", 16, size / 2 + 10, core.hasVisitedFloor(core.getFloorByDirection("left",
				floorId)) ? "#ffffff" : "#ff22ff", "26px Verdana");
		if (core.getFloorByDirection("right", floorId))
			core.fillText("ui", "→", size - 16, size / 2 + 10, core.hasVisitedFloor(core.getFloorByDirection("right",
				floorId)) ? "#ffffff" : "#ff22ff", "26px Verdana");
		if (core.getFloorByDirection("up", floorId))
			core.fillText("ui", "↑", size / 2, 28, core.hasVisitedFloor(core.getFloorByDirection("up",
				floorId)) ? "#ffffff" : "#ff22ff", "26px Verdana");
		if (core.getFloorByDirection("down", floorId))
			core.fillText("ui", "↓", size / 2, size - 4, core.hasVisitedFloor(core.getFloorByDirection("down",
				floorId)) ? "#ffffff" : "#ff22ff", "26px Verdana");
		if (core.getFloorByDirection("top", floorId))
			core.fillText("ui", "上楼", size / 4 + 8, 24, core.hasVisitedFloor(core.getFloorByDirection("top",
				floorId)) ? "#ffffff" : "#ff22ff", "22px " + core.status.globalAttribute.font);
		if (core.getFloorByDirection("bottom", floorId))
			core.fillText("ui", "下楼", size / 4 * 3 - 8, 24, core.hasVisitedFloor(core.getFloorByDirection("bottom",
				floorId)) ? "#ffffff" : "#ff22ff", "22px " + core.status.globalAttribute.font);
		core.fillText("ui", "退10层", size / 4 + 8, size - 8, core.actions._getNextFlyFloor(-1) == page ? "#ff22ff" : "#ffffff",
			"22px " + core.status.globalAttribute.font);
		core.fillText("ui", "进10层", size / 4 * 3 - 8, size - 8, core.actions._getNextFlyFloor(1) == page ? "#ff22ff" : "#ffffff",
			"22px " + core.status.globalAttribute.font);
		core.fillText("ui", "退出", 32, 24, "#ffffff", "22px " + core.status.globalAttribute.font);
		core.fillText("ui", "楼层名", 32, size - 8, "#ffffff", "22px " + core.status.globalAttribute.font);
		core.fillText("ui", "（B）", 16, size - 40, "#ffffff", "22px " + core.status.globalAttribute.font);
		core.createCanvas("mapOnUi", -240, -240, size + 480, size + 480, 150);
		core.drawFlyMap("mapOnUi", 240, 240, size, size, floorId, { fromUser: true, oriFloor: floorId, use3D: flags.use3D });
		if (core.can3D(floorId) && !flags.in3D)
			core.fillText("ui", "3D模式", size - 32, 24, "#ffffff", "20px " + core.status.globalAttribute.font);
		if (flags.in3D)
			core.fillText("ui", "2D模式", size - 32, 24, "#ffffff", "20px " + core.status.globalAttribute.font);
		if (core.can3D(floorId)) core.fillText("ui", "（Z）", size - 16, 56, "#ffffff", "20px " + core.status.globalAttribute.font);
		if (flags.flyTitle) {
			var style = document.getElementById("ui").getContext("2d");
			style.shadowColor = "rgba(0, 0, 0, 1)";
			style.shadowBlur = 5;
			core.fillRect("ui", size / 4, size / 4, size / 2, size / 8, [180, 180, 180, 0.7]);
			core.strokeRect("ui", size / 4, size / 4, size / 2, size / 8, [255, 255, 255, 0.7], 3);
			style.shadowOffsetX = 4;
			style.shadowOffsetY = 2;
			core.fillText("ui", (core.status.maps[floorId] || {}).title, size / 2, size / 16 * 5 + 11, "#ffffff",
				"32px " + core.status.globalAttribute.font);
			style.shadowColor = "none";
			style.shadowBlur = 0;
			style.shadowOffsetX = 0;
			style.shadowOffsetY = 0;
		}
		//***--- 楼传绘制
	};
	////// 楼层传送器界面时的点击操作 //////
	var originClickFly = core.actions._clickFly;
	actions.prototype._clickFly = function (x, y) {
		if (!flags.usePlatFly || core.isReplaying()) return originClickFly.call(core.actions, x, y);
		var page = core.status.event.data;
		var floorId = core.floorIds[page];
		//***--- 点击操作 可以修改的地方只有x,y坐标 其余不可修改
		if (x <= 2 && y <= 1) {
			core.playSound('取消');
			core.deleteCanvas("mapOnUi")
			core.ui.closePanel();
			return;
		}
		// 3D模式
		if (x >= core.__SIZE__ - 2 && y <= 1) {
			if (core.can3D(floorId) && !flags.in3D)
				flags.use3D = true;
			if (flags.in3D) flags.use3D = false;
			core.playSound('光标移动');
			core.ui.drawFly(page);
			return;
		}
		// 显示名称
		if (x <= 1 && y >= core.__SIZE__ - 2) {
			if (flags.flyTitle) flags.flyTitle = false;
			else flags.flyTitle = true;
			core.playSound('光标移动');
			core.ui.drawFly(page);
			return;
		}
		// 飞过去
		if (x > 1 && x < core.__SIZE__ - 1 && y > 1 && y < core.__SIZE__ - 1) {
			if (core.status.maps[core.status.floorId].canFlyFrom &&
				core.status.maps[core.floorIds[core.status.event.data]].canFlyTo &&
				core.hasVisitedFloor(core.floorIds[core.status.event.data])) {
				core.deleteCanvas("mapOnUi");
			}
			core.flyTo(core.floorIds[core.status.event.data]);
		}
		// 前进10层 后退10层
		if (y > core.__SIZE__ - 2 && core.actions._getNextFlyFloor(-1) != page &&
			x >= 2 && x <= Math.floor(core.__SIZE__ / 2) - 2) {
			core.ui.drawFly(this._getNextFlyFloor(-10));
			core.playSound("光标移动");
		}
		if (y > core.__SIZE__ - 2 && core.actions._getNextFlyFloor(1) != page &&
			x >= Math.ceil(core.__SIZE__ / 2) + 1 && x <= core.__SIZE__ - 3) {
			core.ui.drawFly(this._getNextFlyFloor(10));
			core.playSound("光标移动");
		}
		// 获取索引
		function getId(direction) {
			var id = core.getFloorByDirection(direction, floorId);
			for (var i in core.floorIds) {
				if (core.floorIds[i] == id) return parseInt(i);
			}
		}
		// 上下左右和上下楼
		if (x < 1 && core.getFloorByDirection("left", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("left", floorId)) && y >= 2 && y < core.__SIZE__ - 2) {
			core.playSound("光标移动");
			core.drawFly(getId("left"));
		}
		if (x > core.__SIZE__ - 2 && core.getFloorByDirection("right", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("right", floorId)) && y >= 2 && y < core.__SIZE__ - 2) {
			core.playSound("光标移动");
			core.drawFly(getId("right"));
		}
		if (y < 1 && core.getFloorByDirection("up", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("up", floorId)) &&
			x >= Math.floor(core.__SIZE__ / 2) - 1 && x <= Math.ceil(core.__SIZE__ / 2)) {
			core.playSound("光标移动");
			core.drawFly(getId("up"));
		}
		if (y > core.__SIZE__ - 2 && core.getFloorByDirection("down", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("down", floorId)) &&
			x >= Math.floor(core.__SIZE__ / 2) - 1 && x <= Math.ceil(core.__SIZE__ / 2)) {
			core.playSound("光标移动");
			core.drawFly(getId("down"));
		}
		if (y < 1 && x >= 2 && x <= Math.floor(core.__SIZE__ / 2) - 2 &&
			core.getFloorByDirection("top", floorId) && core.hasVisitedFloor(core.getFloorByDirection("top", floorId))) {
			core.playSound("光标移动");
			core.drawFly(getId("top"));
		}
		if (y < 1 && x >= Math.ceil(core.__SIZE__ / 2) + 1 && x <= core.__SIZE__ - 3 &&
			core.getFloorByDirection("bottom", floorId) && core.hasVisitedFloor(core.getFloorByDirection("bottom", floorId))) {
			core.playSound("光标移动");
			core.drawFly(getId("bottom"));
		}
		return;
		//***--- 点击操作
	};
	////// 楼层传送器界面时，按下某个键的操作 //////
	var originKeyDownFly = core.actions._keyDownFly;
	actions.prototype._keyDownFly = function (keycode) {
		if (!flags.usePlatFly || core.isReplaying()) return originKeyDownFly.call(core.actions, keycode);
		var page = core.status.event.data;
		var floorId = core.floorIds[page];
		// 获取索引
		function getId(direction) {
			var id = core.getFloorByDirection(direction, floorId);
			for (var i in core.floorIds) {
				if (core.floorIds[i] == id) return parseInt(i);
			}
		}
		//***--- 按键操作 只可以修改按键的keycode
		if (keycode == 37 && core.getFloorByDirection("left", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("left", floorId))) {
			core.playSound('光标移动');
			core.ui.drawFly(getId("left"));
		} else if (keycode == 38 && core.getFloorByDirection("up", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("up", floorId))) {
			core.playSound('光标移动');
			core.ui.drawFly(getId("up"));
		} else if (keycode == 39 && core.getFloorByDirection("right", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("right", floorId))) {
			core.playSound('光标移动');
			core.ui.drawFly(getId("right"));
		} else if (keycode == 40 && core.getFloorByDirection("down", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("down", floorId))) {
			core.playSound('光标移动');
			core.ui.drawFly(getId("down"));
		} else if (keycode == 33 && core.getFloorByDirection("top", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("top", floorId))) {
			core.playSound('光标移动');
			core.ui.drawFly(getId("top"));
		} else if (keycode == 34 && core.getFloorByDirection("bottom", floorId) &&
			core.hasVisitedFloor(core.getFloorByDirection("bottom", floorId))) {
			core.playSound('光标移动');
			core.ui.drawFly(getId("bottom"));
		} else if (keycode == 90) {
			if (core.can3D(floorId) && !flags.in3D)
				flags.use3D = true;
			if (flags.in3D) flags.use3D = false;
			core.playSound('光标移动');
			core.ui.drawFly(page);
		} else if (keycode == 66) { // 地图名
			if (flags.flyTitle) flags.flyTitle = false;
			else flags.flyTitle = true;
			core.playSound('光标移动');
			core.ui.drawFly(page);
		} else if (keycode == 188 && core.actions._getNextFlyFloor(-1) != page) { // 退10层
			core.ui.drawFly(this._getNextFlyFloor(-10));
			core.playSound("光标移动");
		} else if (keycode == 190 && core.actions._getNextFlyFloor(1) != page) { // 进10层
			core.ui.drawFly(this._getNextFlyFloor(10));
			core.playSound("光标移动");
		}
		return;
		//***--- 按键操作
	};
	////// 楼层传送器界面时，放开某个键的操作 //////
	var originKeyUpFly = core.actions._keyUpFly;
	actions.prototype._keyUpFly = function (keycode) {
		if (!flags.usePlatFly || core.isReplaying()) return originKeyUpFly.call(core.actions, keycode);
		if (keycode == 71 || keycode == 27 || keycode == 88) {
			core.playSound('取消');
			core.deleteCanvas("mapOnUi");
			core.ui.closePanel();
		}
		if (keycode == 13 || keycode == 32 || keycode == 67)
			this._clickFly(this.HSIZE - 1, this.HSIZE - 1);
		return;
	};
	////// 点击楼层传送器时的打开操作 //////
	events.prototype.useFly = function (fromUserAction) {
		if (core.isReplaying()) return;

		// 从“浏览地图”页面：尝试直接传送到该层
		if (core.status.event.id == 'viewMaps') {
			if (!core.hasItem('fly')) {
				core.playSound('操作失败');
				core.drawTip('你没有' + core.material.items['fly'].name, 'fly');
			} else if (!core.canUseItem('fly')) {
				core.playSound('操作失败');
				core.drawTip('无法传送到当前层', 'fly');
			} else {
				core.flyTo(core.status.event.data.floorId);
			}
			return;
		}
		if (!this._checkStatus('fly', fromUserAction, true)) {
			core.deleteCanvas('mapOnUi');
			return;
		}
		if (core.flags.flyNearStair && !core.nearStair()) {
			core.playSound('操作失败');
			core.drawTip("只有在楼梯边才能使用" + core.material.items['fly'].name, 'fly');
			core.unlockControl();
			core.status.event.data = null;
			core.status.event.id = null;
			return;
		}
		if (!core.canUseItem('fly')) {
			core.playSound('操作失败');
			core.drawTip(core.material.items['fly'].name + "好像失效了", 'fly');
			core.unlockControl();
			core.status.event.data = null;
			core.status.event.id = null;
			return;
		}
		core.playSound('打开界面');
		core.useItem('fly', true);
		return;
	};
	// 获取区域平面地图
	this.getFlyMap = function (floorId, fromUser, oriFloor, loop, clearCache) {
		floorId = floorId || core.status.floorId;
		if (floorId == oriFloor && !fromUser) return;
		oriFloor = oriFloor || core.status.floorId;
		if (!floorId) return;
		// 判断是否需要缓存
		function needCache(fromUser) {
			if (fromUser && !core.status.flyMap[floorId]) return true;
			if (!fromUser && !core.status.flyMap.cache[floorId]) return true;
			return false;
		}
		// 缓存，加快运行速率
		if (!core.status.flyMap) core.status.flyMap = {};
		if (!core.status.flyMap.cache) core.status.flyMap.cache = {};
		if (!core.status.layer) core.status.layer = {};
		if (!core.status.layer[floorId]) core.status.layer[floorId] = {};
		if (core.status.flyMap.cache[floorId] && fromUser) delete core.status.flyMap.cache[floorId]
		if (core.status.flyMap[floorId] && !fromUser) delete core.status.flyMap[floorId]
		if (needCache(fromUser) || clearCache) {
			// 初始化
			core.status.flyMap[floorId] = {};
			core.status.flyMap[floorId].thisMap = {};
			core.extractBlocks(floorId);
			core.status.maps[floorId].blocks.forEach(function (block) {
				var id = block.event.id;
				var x = block.x,
					y = block.y;
				var trigger = block.event.trigger;
				if (trigger != "changeFloor" && trigger != "upFloor" && trigger != "downFloor") return;
				// 是箭头且可以切换地图
				var toFloor = block.event.data.floorId;
				// 加入相应位置
				// 箭头
				if (id == leftPortal) {
					core.status.flyMap[floorId].thisMap["left_" + x + "_" + y] = toFloor;
				}
				if (id == upPortal) {
					core.status.flyMap[floorId].thisMap["up_" + x + "_" + y] = toFloor;
				}
				if (id == rightPortal) {
					core.status.flyMap[floorId].thisMap["right_" + x + "_" + y] = toFloor;
				}
				if (id == downPortal) {
					core.status.flyMap[floorId].thisMap["down_" + x + "_" + y] = toFloor;
				}
				// 上下楼
				if (id == upFloor) {
					core.status.flyMap[floorId].thisMap["top_" + x + "_" + y] = toFloor;
					core.status.layer[floorId].top = true;
				}
				if (id == downFloor) {
					core.status.flyMap[floorId].thisMap["bottom_" + x + "_" + y] = toFloor;
					core.status.layer[floorId].bottom = true;
				}
			});
			// 把下几层接着检测出来
			if (fromUser) {
				var usedId = {};
				for (var c = 1; c <= loop; c++) {
					for (var i in core.status.flyMap[floorId].thisMap) {
						var link = core.status.flyMap[floorId].thisMap;
						if (!core.hasVisitedFloor(link[i]) || link[i] instanceof Object || usedId[link[i]]) continue;
						usedId[link[i]] = true;
						var next = core.getFlyMap(link[i], false, oriFloor);
						for (var to in next) {
							if (!core.status.layer[next[to]]) core.status.layer[next[to]] = {};
							core.status.flyMap[floorId].thisMap[i + ',' + to] = next[to];
						}
					}
				}
			}
			// 把先上再下之类的去掉
			if (fromUser) {
				for (var i in core.status.flyMap[floorId].thisMap) {
					var route = i.split(",");
					for (var one = 0; one <= route.length - 2; one++) {
						var step = route[one],
							next = route[one + 1];
						if ((step.startsWith("up") && next.startsWith("down")) ||
							(step.startsWith("down") && next.startsWith("up")) ||
							(step.startsWith("left") && next.startsWith("right")) ||
							(step.startsWith("right") && next.startsWith("left")) ||
							(step.startsWith("top") && next.startsWith("bottom")) ||
							(step.startsWith("bottom") && next.startsWith("top"))) {
							delete core.status.flyMap[floorId].thisMap[i];
						}
					}
				}
			}
			// 非当前层不能存此类缓存
			if (!fromUser) {
				core.status.flyMap.cache = {};
				core.status.flyMap.cache[floorId] = {};
				core.status.flyMap.cache[floorId].thisMap = core.status.flyMap[floorId].thisMap;
				delete core.status.flyMap[floorId];
			}
			return fromUser ? core.status.flyMap[floorId].thisMap : core.status.flyMap.cache[floorId].thisMap;
		} else { // 直接使用缓存
			return fromUser ? core.status.flyMap[floorId].thisMap : core.status.flyMap.cache[floorId].thisMap;
		}
	};
	this.can3D = function (floorId) {
		var map = core.getFlyMap(floorId, true, floorId);
		for (var route in map) {
			if (route.indexOf("top") >= 0 || route.indexOf("bottom") >= 0)
				return true;
		}
		return false;
	};
	// 绘制地图
	this.drawFlyMap = function (ctx, x, y, width, height, floorId, config) {
		// 初始化配置项
		//***--- 初始化 可以修改 || 后的默认值 参数说明在开头的高深区域中
		var fromUser = config.fromUser || false,
			oriFloor = config.oriFloor || core.status.floorId,
			scale = config.scale || null,
			interval = config.interval || (width / 24),
			noErase = config.noErase || false,
			fromMini = config.fromMini || false,
			loop = config.loop || defaultLoop,
			opacity = config.opacity || defaultOpacity,
			layer = config.layer || 0,
			use3D = config.use3D || false,
			clearCache = config.clearCache || false,
			map = config.map || null;
		//***--- 初始化
		map = map || core.getFlyMap(floorId, fromUser, oriFloor, loop, clearCache);
		floorId = floorId || core.status.floorId;
		if (!floorId) return;
		// 检测是否需要3D绘图
		if (!fromMini && use3D) {
			for (var route in map) {
				if (route.indexOf("top") >= 0 || route.indexOf("bottom") >= 0) {
					config.map = map;
					return core.draw3DFlyMap(ctx, x, y, width, height, floorId, config);
				}
			}
		}
		if (layer != 0 && !use3D) {
			var canLayer = false
			for (var route in map) {
				if (route.indexOf("top") >= 0 || route.indexOf("bottom") >= 0) {
					canLayer = true;
					break;
				}
			}
			if (!canLayer) layer = 0;
		}
		flags.in3D = false;
		// 初始化
		var userScale = true;
		var newCreate = false;
		if (!scale) {
			userScale = false;
			scale = scale || defaultScale;
		}
		if (!core.dymCanvas[ctx]) {
			core.createCanvas(ctx, x, y, width, height, 140);
			newCreate = true;
		}
		x = x || 0;
		y = y || 0;
		// 获得canvas属性
		width = width || document.getElementById(ctx).width;
		height = height || document.getElementById(ctx).height;
		var oLeft = document.getElementById(ctx).offsetLeft / core.domStyle.scale,
			oTop = document.getElementById(ctx).offsetTop / core.domStyle.scale;
		// 重置大地图和楼传地图的canvas位置
		if (ctx == "mapOnUi") core.relocateCanvas("mapOnUi", -240, -240);
		if (!noErase)
			core.clearMap(ctx);
		var horCenter = Math.floor(width / 2),
			uprCenter = Math.floor(height / 2);
		if (!newCreate) {
			horCenter += x;
			uprCenter += y;
		}
		var centerX = horCenter,
			centerY = uprCenter;
		var left = centerX,
			right = centerX,
			up = centerY,
			down = centerY;
		var used = {};
		var haveLayer = {};
		var nx = horCenter,
			ny = uprCenter;
		// 先把所在楼层绘制了
		if (layer == 0) {
			var nw = core.status.maps[floorId].width * 2 * scale,
				nh = core.status.maps[floorId].height * 2 * scale;
			core.setAlpha(ctx, 1);
			core.fillRect(ctx, centerX - nw / 2, centerY - nh / 2, nw, nh, "#000000");
			core.strokeRect(ctx, centerX - nw / 2, centerY - nh / 2, nw, nh, "#ffff22", 3 * scale);
			// 当前层上下楼显示
			if (!haveLayer[floorId]) {
				core.setAlpha(ctx, 1);
				var needLayer = core.status.layer[floorId];
				if (needLayer.top && needLayer.bottom) {
					core.drawIcon(ctx, "upFloor", centerX - core.__SIZE__ * scale,
						centerY - core.__SIZE__ * scale, core.__SIZE__ * scale, core.__SIZE__ * scale);
					core.drawIcon(ctx, "downFloor", centerX - nw / 2 + core.__SIZE__ * scale,
						centerY - nh / 2 + core.__SIZE__ * scale, core.__SIZE__ * scale, core.__SIZE__ * scale);
				}
				if (needLayer.top && !needLayer.bottom) {
					core.drawIcon(ctx, "upFloor", centerX - Math.min(nw, nh) / 2, centerY - Math.min(nw, nh) / 2,
						Math.min(nw, nh), Math.min(nw, nh));
				}
				if (!needLayer.top && needLayer.bottom) {
					core.drawIcon(ctx, "downFloor", centerX - Math.min(nw, nh) / 2, centerY - Math.min(nw, nh) / 2,
						Math.min(nw, nh), Math.min(nw, nh));
				}
				haveLayer[floorId] = true;
			}
			// 四侧最远位置
			if (left > centerX - nw / 2) left = centerX - nw / 2;
			if (right < centerX + nw / 2) right = centerX + nw / 2;
			if (down < centerY + nh / 2) down = centerY + nh / 2;
			if (up > centerY - nh / 2) up = centerY - nh / 2;
		}
		core.setAlpha(ctx, opacity);
		for (var route in map) { // 绘制楼层和线条
			var rouArr = route.split(",");
			// 检索路线及画线
			// 初始化
			centerX = nx;
			centerY = ny;
			var nowFloor = floorId || core.status.floorId;
			var nowLayer = 0;
			for (var one in rouArr) { // 一个一个检测
				var step = rouArr[one].split("_");
				var cx = step[1],
					cy = step[2];
				// 获得当前图块
				core.getMapBlocksObj(nowFloor, true);
				var nowBlock = core.status.mapBlockObjs[nowFloor][cx + ',' + cy];
				if (!nowBlock) continue;
				var toLoc = nowBlock.event.data.loc,
					toFloor = nowBlock.event.data.floorId;
				var needLayer = core.status.layer[toFloor];
				// 当前层宽度和高度
				var nw = core.status.maps[nowFloor].width * 2 * scale,
					nh = core.status.maps[nowFloor].height * 2 * scale;
				// 目标层宽度和高度
				var tw = core.status.maps[toFloor].width * 2 * scale,
					th = core.status.maps[toFloor].height * 2 * scale;
				// 将当前层变为toFloor
				nowFloor = toFloor;
				// 超范围不画
				if ((centerX > oLeft + x + width || centerX < oLeft + x || centerY > oTop + y + height ||
						centerY < oTop + y) && userScale && !fromMini && ctx != "mapOnUi") continue;
				// 绘制toFloor层
				core.setAlpha(ctx, opacity);
				// 确定center 根据箭头自适配 同时绘制线条 我已经看不懂了
				if (!use3D && (step[0] == "top" || step[0] == "bottom") && layer == 0) break;
				if (step[0] == "top") nowLayer++;
				if (step[0] == "bottom") nowLayer--;
				if (step[0] == 'left') {
					var shouldTo = th / 2,
						realTo = toLoc[1] * 2 * scale;
					var shouldFrom = nh / 2,
						realFrom = step[2] * 2 * scale;
					if (nowLayer == layer) {
						core.drawLine(ctx, centerX - nw / 2, centerY + realFrom - shouldFrom,
							centerX - nw / 2 - interval, centerY + realFrom - shouldFrom, "#ffffff", 5 * scale);
						core.drawLine(ctx, centerX - nw / 2, centerY + realFrom - shouldFrom,
							centerX - nw / 2 - interval, centerY + realFrom - shouldFrom, "#000000", 2 * scale);
					}
					centerX -= nw / 2 + tw / 2 + interval;
					centerY += shouldTo - realTo + realFrom - shouldFrom;
				}
				if (step[0] == 'right') {
					var shouldTo = th / 2,
						realTo = toLoc[1] * 2 * scale;
					var shouldFrom = nh / 2,
						realFrom = step[2] * 2 * scale;
					if (nowLayer == layer) {
						core.drawLine(ctx, centerX + nw / 2, centerY + realFrom - shouldFrom,
							centerX + nw / 2 + interval, centerY + realFrom - shouldFrom, "#ffffff", 5 * scale);
						core.drawLine(ctx, centerX + nw / 2, centerY + realFrom - shouldFrom,
							centerX + nw / 2 + interval, centerY + realFrom - shouldFrom, "#000000", 2 * scale);
					}
					centerX += nw / 2 + tw / 2 + interval;
					centerY += shouldTo - realTo + realFrom - shouldFrom;
				}
				if (step[0] == 'up') {
					var shouldTo = tw / 2,
						realTo = toLoc[0] * 2 * scale;
					var shouldFrom = nw / 2,
						realFrom = step[1] * 2 * scale;
					if (nowLayer == layer) {
						core.drawLine(ctx, centerX + realFrom - shouldFrom, centerY - nh / 2,
							centerX + realFrom - shouldFrom, centerY - nh / 2 - interval, "#ffffff", 5 * scale);
						core.drawLine(ctx, centerX + realFrom - shouldFrom, centerY - nh / 2,
							centerX + realFrom - shouldFrom, centerY - nh / 2 - interval, "#000000", 2 * scale);
					}
					centerY -= nh / 2 + th / 2 + interval;
					centerX += shouldTo - realTo + realFrom - shouldFrom;
				}
				if (step[0] == 'down') {
					var shouldTo = tw / 2,
						realTo = toLoc[0] * 2 * scale;
					var shouldFrom = nw / 2,
						realFrom = step[1] * 2 * scale;
					if (nowLayer == layer) {
						core.drawLine(ctx, centerX + realFrom - shouldFrom, centerY + nh / 2,
							centerX + realFrom - shouldFrom, centerY + nh / 2 + interval, "#ffffff", 5 * scale);
						core.drawLine(ctx, centerX + realFrom - shouldFrom, centerY + nh / 2,
							centerX + realFrom - shouldFrom, centerY + nh / 2 + interval, "#000000", 2 * scale);
					}
					centerY += nh / 2 + th / 2 + interval;
					centerX += shouldTo - realTo + realFrom - shouldFrom;
				}
				// 只有和目标层高度相同时才绘制
				if (nowLayer != layer) continue;
				// 超范围的不画
				if ((centerX > oLeft + x + width || centerX < oLeft + x || centerY > oTop + y + height ||
						centerY < oTop + y) && userScale && !fromMini && ctx != "mapOnUi") continue;
				// 四侧最远位置
				if (left > centerX - tw / 2) left = centerX - tw / 2;
				if (right < centerX + tw / 2) right = centerX + tw / 2;
				if (down < centerY + th / 2) down = centerY + th / 2;
				if (up > centerY - th / 2) up = centerY - th / 2;
				// 画过了不画
				if (used[toFloor]) continue;
				used[toFloor] = true;
				// 画地图格
				if (core.hasVisitedFloor(toFloor)) {
					core.fillRect(ctx, centerX - tw / 2, centerY - th / 2, tw, th, "#000000");
					core.strokeRect(ctx, centerX - tw / 2, centerY - th / 2, tw, th, "#ffffff", 3 * scale);
				} else {
					core.fillRect(ctx, centerX - tw / 2, centerY - th / 2, tw, th, "#ff22ff");
					core.strokeRect(ctx, centerX - tw / 2, centerY - th / 2, tw, th, "#ffffff", 3 * scale);
					break;
				}
				// 上下楼显示
				if (haveLayer[toFloor]) continue;
				core.setAlpha(ctx, opacity);
				if (needLayer.top && needLayer.bottom) {
					core.drawIcon(ctx, "upFloor", centerX - core.__SIZE__ * scale,
						centerY - core.__SIZE__ * scale, core.__SIZE__ * scale, core.__SIZE__ * scale);
					core.drawIcon(ctx, "downFloor", centerX - tw / 2 + core.__SIZE__ * scale,
						centerY - th / 2 + core.__SIZE__ * scale, core.__SIZE__ * scale, core.__SIZE__ * scale);
				}
				if (needLayer.top && !needLayer.bottom) {
					core.drawIcon(ctx, "upFloor", centerX - Math.min(tw, th) / 2, centerY - Math.min(tw, th) / 2,
						Math.min(tw, th), Math.min(tw, th));
				}
				if (!needLayer.top && needLayer.bottom) {
					core.drawIcon(ctx, "downFloor", centerX - Math.min(tw, th) / 2, centerY - Math.min(tw, th) / 2,
						Math.min(tw, th), Math.min(tw, th));
				}
				haveLayer[toFloor] = true;
			}
		}
		// 自动缩放
		if ((right - left > core.__PIXELS__ - 64 || down - up > core.__PIXELS__ - 64) && !userScale && !fromMini) {
			scale = 1 / (Math.max(right - left, down - up) / (core.__PIXELS__ - 64));
			var con = { fromUser: fromUser, oriFloor: oriFloor, scale: scale, interval: interval * scale, layer: layer, opacity: opacity, loop: loop };
			return core.drawFlyMap(ctx, x, y, width, height, floorId, con);
		}
		// 大地图和楼层地图自适配定位
		if (ctx == "mapOnUi" && !fromMini && (left - nx < -128 || right - nx > 128 ||
				up - ny < -128 || down - ny > 128)) {
			core.relocateCanvas("mapOnUi", -240 + (-left - right + 2 * nx) / 2, -240 + (-up - down + 2 * ny) / 2);
		}
	};
	// 3D绘图
	this.draw3DFlyMap = function (ctx, x, y, width, height, floorId, config) {
		// 初始化配置项
		//***--- 初始化 同上一个初始化
		var fromUser = config.fromUser || false,
			oriFloor = config.oriFloor || core.status.floorId,
			scale = config.scale || null,
			interval = config.interval || (width / 24),
			deltaH = config.deltaH || (height / 8),
			noErase = config.noErase || false,
			fromMini = config.fromMini || false,
			loop = config.loop || defaultLoop,
			opacity = config.opacity || defaultOpacity,
			minorAlpha = config.minorAlpha || defaultMinorAlpha,
			reLeft = config.reLeft || -240,
			reTop = config.reTop || -240,
			clearCache = config.clearCache || false,
			map = config.map || null;
		//***--- 初始化
		map = map || core.getFlyMap(floorId, fromUser, oriFloor, loop, clearCache);
		// 当前层是否一个人在一层
		var alone = true;
		for (var route in map) {
			if (route.startsWith("left") || route.startsWith("right") ||
				route.startsWith("up") || route.startsWith("down")) {
				alone = false;
				break;
			}
		}
		// 是则增加一个先上再下的路径
		if (alone) {
			for (var route in map) {
				if (route.startsWith("top")) {
					var first = route.split(",")[0].split("_");
					break;
				}
			}
			var success = false;
			core.getMapBlocksObj(nowFloor, true);
			var nowBlock = core.status.mapBlockObjs[floorId][first[1] + "," + first[2]];
			var toFloor = nowBlock.event.data.floorId;
			core.extractBlocks(toFloor);
			core.status.maps[toFloor].blocks.forEach(function (block) {
				var id = block.event.id;
				var x = block.x,
					y = block.y;
				var trigger = block.event.trigger;
				if (trigger != "changeFloor") return;
				if (id == "downFloor") {
					map["top_" + first[1] + "_" + first[2] + "," + "bottom_" + x + "_" + y] = floorId;
					success = true;
					return;
				}
			});
			// 添加先上再下失败 尝试先下再上
			if (!success) {
				for (var route in map) {
					if (route.startsWith("bottom")) {
						var first = route.split(",")[0].split("_");
						break;
					}
				}
				var nowBlock = core.status.mapBlockObjs[floorId][first[1] + "," + first[2]];
				var toFloor = nowBlock.event.data.floorId;
				core.extractBlocks(toFloor);
				core.status.maps[toFloor].blocks.forEach(function (block) {
					var id = block.event.id;
					var x = block.x,
						y = block.y;
					var trigger = block.event.trigger;
					if (trigger != "changeFloor") return;
					if (id == "upFloor") {
						map["bottom_" + first[1] + "_" + first[2] + "," + "top_" + x + "_" + y] = floorId;
						success = true;
						return;
					}
				});
			}
		}
		floorId = floorId || core.status.floorId;
		if (!floorId) return;
		flags.in3D = true;
		// 初始化
		// 获得排序过的楼层路径
		map = core.sortFloor(map);
		map = map.map;
		var userScale = true;
		var newCreate = false;
		if (!scale) {
			userScale = false;
			scale = scale || defaultScale;
		}
		if (!core.dymCanvas[ctx]) {
			core.createCanvas(ctx, x, y, width, height, 140);
			newCreate = true;
		}
		x = x || 0;
		y = y || 0;
		// 获得canvas属性
		width = width || document.getElementById(ctx).width;
		height = height || document.getElementById(ctx).height;
		// 重置canvas位置
		core.relocateCanvas(ctx, reLeft, reTop);
		if (!noErase)
			core.clearMap(ctx);
		var horCenter = Math.floor(width / 2),
			uprCenter = Math.floor(height / 2);
		if (!newCreate) {
			horCenter += x;
			uprCenter += y;
		}
		// 单元格的中心点 即水平线中点处
		var centerX = horCenter,
			centerY = uprCenter;
		var left = centerX,
			right = centerX,
			up = centerY,
			down = centerY;
		var used = {};
		var nx = horCenter,
			ny = uprCenter;
		// 开始绘制
		for (var i = 0; i < map.length; i++) {
			var route = map[i][0];
			var nowLayer = map[i][1];
			var everyLayer = 0;
			route = route.split(",");
			// 每条路线初始化
			centerX = horCenter;
			centerY = uprCenter;
			centerX += core.status.maps[floorId].height * scale * Math.SQRT2 / 4;
			if (flags.viewingLayer) {
				centerY += deltaH * flags.viewingLayer;
			}
			var nowFloor = floorId || core.status.floorId;
			for (var one = 0; one < route.length; one++) {
				var step = route[one].split("_");
				var cx = step[1],
					cy = step[2];
				// 检测高度，是否与nowLayer一致 不一致在处理完center以后不绘制
				if (step[0] == "top") everyLayer++;
				if (step[0] == "bottom") everyLayer--;
				// 获得当前图块
				core.getMapBlocksObj(nowFloor, true);
				var nowBlock = core.status.mapBlockObjs[nowFloor][cx + ',' + cy];
				if (!nowBlock) continue;
				var toLoc = nowBlock.event.data.loc,
					toFloor = nowBlock.event.data.floorId;
				// 当前层宽度和高度
				// 斜二测画法
				var nw = core.status.maps[nowFloor].width * 2 * scale,
					nh = core.status.maps[nowFloor].height * scale * Math.SQRT2 / 2;
				// 目标层宽度和高度
				var tw = core.status.maps[toFloor].width * 2 * scale,
					th = core.status.maps[toFloor].height * scale * Math.SQRT2 / 2;
				if (!(toLoc instanceof Array)) {
					toLoc = [Math.floor(tw / 4 / scale), Math.floor(th / 4 / scale)];
				}
				// 绘制当前层
				if (nowLayer == 0 && !used[floorId]) {
					core.setAlpha(ctx, 1);
					used[floorId] = true;
					var nowW = core.status.maps[floorId].width * 2 * scale;
					var nowH = core.status.maps[floorId].height * scale * Math.SQRT2 / 2;
					var nodes = [
						[centerX - nowW / 2 - nowH / 2, centerY + nowH / 2],
						[centerX + nowW / 2 - nowH / 2, centerY + nowH / 2],
						[centerX + nowW / 2 + nowH / 2, centerY - nowH / 2],
						[centerX - nowW / 2 + nowH / 2, centerY - nowH / 2]
					];
					core.fillPolygon(ctx, nodes, "#000000");
					core.strokePolygon(ctx, nodes, "#ffff22", 1.5 * scale);
					// 四侧最远位置
					if (left > centerX - nw / 2 - nh / 2 && nowLayer == (flags.viewingLayer || 0)) left = centerX - nw / 2 - nh / 2;
					if (right < centerX + nw / 2 + nh / 2 && nowLayer == (flags.viewingLayer || 0)) right = centerX + nw / 2 + nh / 2;
				}
				// 将当前层变为toFloor
				var fromFloor = nowFloor;
				nowFloor = toFloor;
				// 计算center 画同层间的线 我已经看不懂了
				// 设置不透明度
				if (nowLayer == (flags.viewingLayer || 0)) {
					core.setAlpha(ctx, opacity);
				} else {
					core.setAlpha(ctx, minorAlpha * Math.max(0, 1 - 0.34 * Math.abs(nowLayer - (flags.viewingLayer || 0))));
				}
				if (step[0] == "left") {
					var shouldFrom = nh / 2,
						realFrom = cy * scale * Math.SQRT2 / 2;
					var shouldTo = th / 2,
						realTo = toLoc[1] * scale * Math.SQRT2 / 2;
					if (everyLayer == nowLayer && !used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy])
						core.drawLine(ctx, centerX - nw / 2 + nh / 2 - realFrom, centerY + realFrom - shouldFrom,
							centerX - nw / 2 - interval + nh / 2 - realFrom, centerY + realFrom - shouldFrom, "#ffffff", 2 * scale);
					centerX -= nw / 2 + tw / 2 + interval + shouldTo - realTo + realFrom - shouldFrom;
					centerY += shouldTo - realTo + realFrom - shouldFrom;
				}
				if (step[0] == "right") {
					var shouldFrom = nh / 2,
						realFrom = cy * scale * Math.SQRT2 / 2;
					var shouldTo = th / 2,
						realTo = toLoc[1] * scale * Math.SQRT2 / 2;
					if (nowLayer == everyLayer && !used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy])
						core.drawLine(ctx, centerX + nw / 2 + nh / 2 - realFrom, centerY + realFrom - shouldFrom,
							centerX + nw / 2 + interval + nh / 2 - realFrom, centerY + realFrom - shouldFrom, "#ffffff", 2 * scale);
					centerX += nw / 2 + tw / 2 + interval - (shouldTo - realTo + realFrom - shouldFrom);
					centerY += shouldTo - realTo + realFrom - shouldFrom;
				}
				if (step[0] == "up") {
					var shouldTo = tw / 2,
						realTo = toLoc[0] * scale * 2;
					var shouldFrom = nw / 2,
						realFrom = cx * scale * 2;
					if (nowLayer == everyLayer && !used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy])
						core.drawLine(ctx, centerX + realFrom - shouldFrom + nh / 2, centerY - nh / 2, centerX + realFrom -
							shouldFrom + interval * Math.SQRT2 / 4 + nh / 2, centerY - nh / 2 - interval * Math.SQRT2 / 4, "#ffffff", 2 * scale);
					centerY -= nh / 2 + th / 2 + interval * Math.SQRT2 / 4;
					centerX += shouldTo - realTo + realFrom - shouldFrom + (nh / 2 + th / 2 + interval * Math.SQRT2 / 4);
				}
				if (step[0] == "down") {
					var shouldTo = tw / 2,
						realTo = toLoc[0] * scale * 2;
					var shouldFrom = nw / 2,
						realFrom = cx * scale * 2;
					if (nowLayer == everyLayer && !used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy])
						core.drawLine(ctx, centerX + realFrom - shouldFrom - nh / 2, centerY + nh / 2, centerX + realFrom -
							shouldFrom - interval * Math.SQRT2 / 4 - nh / 2, centerY + nh / 2 + interval * Math.SQRT2 / 4, "#ffffff", 2 * scale);
					centerY += nh / 2 + th / 2 + interval * Math.SQRT2 / 4;
					centerX += shouldTo - realTo + realFrom - shouldFrom - (nh / 2 + th / 2 + interval * Math.SQRT2 / 4);
				}
				if (step[0] == "top") {
					centerY -= deltaH;
				}
				if (step[0] == "bottom") {
					centerY += deltaH;
				}
				if (everyLayer == nowLayer && step[0] != "top" && step[0] != "bottom") {
					used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy] = true;
					used[toFloor + "_" + fromFloor + "_" + toLoc[0] + "_" + toLoc[1]] = true;
				}
				if (everyLayer != nowLayer) continue;
				// 四侧最远位置
				if (!flags.viewingLayer) {
					if (left > centerX - tw / 2 - th / 2 && nowLayer == (flags.viewingLayer || 0)) left = centerX - tw / 2 - th / 2;
					if (right < centerX + tw / 2 + th / 2 && nowLayer == (flags.viewingLayer || 0)) right = centerX + tw / 2 + th / 2;
					if (down < centerY + th / 2 && nowLayer == (flags.viewingLayer || 0)) down = centerY + th / 2;
					if (up > centerY - th / 2 && nowLayer == (flags.viewingLayer || 0)) up = centerY - th / 2;
				}
				// 不同高度层之间的连线
				if (!used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy]) {
					core.setAlpha(ctx, opacity * Math.max(0, 1 - 0.34 * Math.abs(nowLayer - (flags.viewingLayer || 0))));
					if (step[0] == "top") {
						core.drawLine(ctx, centerX, centerY + deltaH, centerX, centerY, "#ffffff", 5 * scale);
						core.drawLine(ctx, centerX, centerY + deltaH, centerX, centerY, "#000000", 2 * scale);
						used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy] = true;
						used[toFloor + "_" + fromFloor + "_" + toLoc[0] + "_" + toLoc[1]] = true;
					}
				}
				if (!used[toFloor]) {
					used[toFloor] = true;
					// 设置不透明度
					if (nowLayer == (flags.viewingLayer || 0)) {
						core.setAlpha(ctx, opacity);
					} else {
						core.setAlpha(ctx, minorAlpha * Math.max(0, 1 - 0.34 * Math.abs(nowLayer - (flags.viewingLayer || 0))));
					}
					// 画地图
					var nodes = [
						[centerX - tw / 2 - th / 2, centerY + th / 2], // 左下
						[centerX + tw / 2 - th / 2, centerY + th / 2], // 右下
						[centerX + tw / 2 + th / 2, centerY - th / 2], // 右上
						[centerX - tw / 2 + th / 2, centerY - th / 2] // 左上
					];
					if (core.hasVisitedFloor(toFloor)) {
						core.fillPolygon(ctx, nodes, "#000000");
					} else {
						core.fillPolygon(ctx, nodes, "#ff22ff");
					}
					core.strokePolygon(ctx, nodes, "#ffffff", 1.5 * scale);
				}
				// 不同高度层之间的连线
				if (!used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy]) {
					if (step[0] == "bottom") {
						core.drawLine(ctx, centerX, centerY, centerX, centerY - deltaH, "#ffffff", 5 * scale);
						core.drawLine(ctx, centerX, centerY, centerX, centerY - deltaH, "#000000", 2 * scale);
						used[fromFloor + "_" + toFloor + "_" + cx + "_" + cy] = true;
						used[toFloor + "_" + fromFloor + "_" + toLoc[0] + "_" + toLoc[1]] = true;
					}
				}
			}
		}
		// 自动缩放
		if ((right - left > core.__PIXELS__ - 64 || down - up > core.__PIXELS__ - 64) && !userScale && !fromMini) {
			scale = 1 / (Math.max(right - left, down - up) / (core.__PIXELS__ - 64));
			var con = { fromUser: fromUser, oriFloor: oriFloor, scale: scale, interval: interval * scale, opacity: opacity, loop: loop, use3D: true };
			return core.draw3DFlyMap(ctx, x, y, width, height, floorId, con);
		}
		// 大地图和楼层地图自适配定位
		if (ctx == "mapOnUi")
			core.relocateCanvas("mapOnUi", -240 + (-left - right + 2 * nx) / 2, -240 + (-up - down + 2 * ny) / 2);
	};
	// 不同高度楼层排序
	this.sortFloor = function (map) {
		map = map || core.getFlyMap(null, true);
		var totalLayer = 1,
			topLayer = 0,
			bottomLayer = 0,
			nowLayer = 0;
		// 拆分map
		for (var i in map) {
			var route = i.split(",");
			nowLayer = 0;
			for (var one in route) {
				var step = route[one].split("_");
				// 层数处理 并记录每一层的层数
				if (step[0] == "top") {
					nowLayer++;
					map[i] = nowLayer + "_" + map[i];
					if (nowLayer > topLayer) topLayer = nowLayer;
				}
				if (step[0] == "bottom") {
					nowLayer--;
					map[i] = nowLayer + "_" + map[i];
					if (nowLayer < bottomLayer) bottomLayer = nowLayer;
				}
			}
		}
		// 总层数
		totalLayer = topLayer - bottomLayer + 1;
		// 按楼层高度由低到高排序
		// 先变成数组
		var mapArr = [];
		for (var one in map) {
			mapArr.push([one, parseInt(map[one]) || 0]);
		}
		// 再sort
		mapArr.sort(function (a, b) { return a[1] - b[1]; });
		return { map: mapArr, totalLayer: totalLayer, top: topLayer, bottom: bottomLayer };
	};
	// 由方向获得楼层坐标
	this.getFloorByDirection = function (direction, floorId) {
		floorId = floorId || core.status.floorId;
		var route = core.getFlyMap(floorId);
		for (var step in route) {
			if (step.indexOf(direction) >= 0) {
				return route[step];
			}
		}
		return null;
	};
	////// 转换楼层结束的事件 检查小地图 //////
	var originAfterChangeFloor = core.events.afterChangeFloor;
	events.prototype.afterChangeFloor = function (floorId) {
		if (!flags.__useMinimap__ || core.isReplaying()) {
			core.deleteCanvas("minimap");
			core.deleteCanvas("mapArrow");
			core.unregisterAction("ondown", "closeMinimap");
			core.unregisterAction("ondown", "openMinimap");
			return originAfterChangeFloor.call(core.events, floorId);
		}
		if (main.mode != 'play') return;
		this.eventdata.afterChangeFloor(floorId);
		// 防止小地图出问题
		core.unregisterAction("ondown", "closeMinimap");
		core.unregisterAction("ondown", "openMinimap");
		// 切换小地图
		core.checkMinimap(true, true);
		return;
	};
	////// 瞬间移动 检查小地图 //////
	var originMoveDirectly = core.control.moveDirectly;
	control.prototype.moveDirectly = function (destX, destY, ignoreSteps) {
		if (!flags.__useMinimap__ || core.isReplaying())
			return originMoveDirectly.call(core.control, destX, destY, ignoreSteps);
		var canMoveDirectly = this.controldata.moveDirectly(destX, destY, ignoreSteps);
		if (canMoveDirectly) core.checkMinimap();
		return canMoveDirectly;
	};
	////// 每移动一格后执行的事件 检查小地图 //////
	var originMoveOneStep = core.control.moveOneStep;
	control.prototype.moveOneStep = function (callback) {
		if (!flags.__useMinimap__ || core.isReplaying())
			return originMoveOneStep.call(core.control, callback);
		this.controldata.moveOneStep(callback);
		core.checkMinimap();
	};
	// 检查小地图开闭情况 改变小地图位置
	this.checkMinimap = function (fromUser, reDraw) {
		if (!flags.__useMinimap__ || core.isReplaying()) {
			core.unregisterAction("ondown", "closeMinimap");
			core.unregisterAction("ondown", "openMinimap");
			core.deleteCanvas("mapArrow");
			core.deleteCanvas("minimap");
			return;
		}
		reDraw = reDraw || false;
		// 是否重绘
		if (reDraw) {
			if (flags.minimap) core.drawMinimap(flags.__onLeft__);
			else core.drawClosedMap(flags.__onLeft__);
		}
		var hx = core.status.hero.loc.x;
		var opened = flags.minimap;
		var onLeft = hx >= Math.ceil(core.__SIZE__ / 3 * 2);
		fromUser = fromUser || false; // 开关小地图相关
		if (!flags.__onLeft__) flags.__onLeft__ = false;
		// 如果地图上没有小地图 画到右边
		if (!core.dymCanvas.mapArrow && !core.dymCanvas.minimap && !reDraw) {
			if (flags.minimap) core.drawMinimap();
			else core.drawClosedMap();
			flags.__onLeft__ = false;
		}
		// 人物在中间 不执行
		if (hx >= Math.ceil(core.__SIZE__ / 3 + core.bigmap.offsetX / 32) &&
			hx <= Math.floor(core.__SIZE__ / 3 * 2 + core.bigmap.offsetX / 32)) return;
		// 重定位画布 和 翻转
		// 挪到右边
		if (!onLeft && (flags.__onLeft__ || fromUser)) {
			flags.__onLeft__ = false;
			if (opened) {
				core.relocateCanvas("minimap", core.__PIXELS__ - 120, 0);
				core.relocateCanvas("mapArrow", core.__PIXELS__ - 140, 0);
				document.getElementById('mapArrow').style.transform = 'none';
			} else {
				core.relocateCanvas("minimap", core.__PIXELS__, 0);
				core.relocateCanvas("mapArrow", core.__PIXELS__ - 20, 0);
				document.getElementById('mapArrow').style.transform = 'none';
			}
		}
		// 挪到左边
		if (onLeft && (!flags.__onLeft__ || fromUser)) {
			flags.__onLeft__ = true;
			if (opened) {
				core.relocateCanvas("minimap", 0, 0);
				core.relocateCanvas("mapArrow", 120, 0);
				document.getElementById('mapArrow').style.transform = 'rotateY(180deg)';
			} else {
				core.relocateCanvas("minimap", -120, 0);
				core.relocateCanvas("mapArrow", 0, 0);
				document.getElementById('mapArrow').style.transform = 'rotateY(180deg)';
			}
		}
	};
	// 点击小地图的action
	this.registerMinimapAction = function (open) {
		if (!open) {
			core.registerAction("ondown", "closeMinimap", function (x, y, px, py) {
				if (!flags.__onLeft__) {
					if (px >= core.__PIXELS__ - 140 && px <= core.__PIXELS__ - 120 &&
						py >= 0 && py <= 120) {
						core.closeMinimap();
						core.unregisterAction("ondown", "closeMinimap");
						return true;
					}
					if (px >= core.__PIXELS__ - 120 && py <= 120) {
						core.playSound("打开界面");
						core.drawTotalMap();
						return true;
					}
				} else {
					if (px >= 120 && px <= 140 && py >= 0 && py <= 120) {
						core.closeMinimap();
						core.unregisterAction("ondown", "closeMinimap");
						return true;
					}
					if (px <= 120 && py <= 120) {
						core.playSound("打开界面");
						core.drawTotalMap();
						return true;
					}
				}
			}, 10);
		} else {
			core.registerAction("ondown", "openMinimap", function (x, y, px, py) {
				if (!flags.__onLeft__) {
					if (px >= core.__PIXELS__ - 20 && py <= 120) {
						core.openMinimap();
						core.unregisterAction("ondown", "openMinimap");
						return true;
					}
				} else {
					if (px <= 20 && py <= 120) {
						core.openMinimap();
						core.unregisterAction("ondown", "openMinimap");
						return true;
					}
				}
			}, 10);
		}
	};
	// 地图上的小地图
	this.drawMinimap = function (toLeft) {
		if (!flags.__useMinimap__) {
			core.deleteCanvas("mapArrow");
			core.deleteCanvas("minimap");
			return;
		}
		var scale = 1.3 / core.status.thisMap.width * 15 * (flags.userScale || 1);
		if (1.3 / core.status.thisMap.height * 15 * (flags.userScale || 1) < scale)
			scale = 1.3 / core.status.thisMap.height * 15 * (flags.userScale || 1);
		// 绘制
		core.createCanvas("minimap", core.__PIXELS__ - 120, 0, 120, 120, 100);
		core.createCanvas("mapArrow", core.__PIXELS__ - 140, 0, 20, 120, 100);
		if (toLeft) {
			core.relocateCanvas("minimap", 0, 0);
			core.relocateCanvas("mapArrow", 120, 0);
			document.getElementById('mapArrow').style.transform = 'rotateY(180deg)';
		}
		core.clearMap("minimap");
		core.clearMap("mapArrow");
		// 黑色底
		core.fillRect("minimap", 0, 0, 120, 120, [0, 0, 0, 0.6]);
		var config = { fromUser: true, oriFloor: core.status.floorId, scale: scale, interval: 10, noErase: true, fromMini: true };
		core.drawFlyMap("minimap", 0, 0, 120, 120, core.status.floorId, config);
		// 向右箭头
		core.fillRect("mapArrow", 0, 0, 20, 120, [230, 230, 230, 0.9]);
		core.drawLine("mapArrow", 0, 20, 20, 20, [100, 100, 100, 0.9], 2);
		core.drawLine("mapArrow", 0, 100, 20, 100, [100, 100, 100, 0.9], 2);
		core.setTextAlign("mapArrow", "center");
		core.fillText("mapArrow", ">", 10, 67, [100, 100, 100, 0.9], "20px Verdana");
		core.registerMinimapAction(false);
	};
	// 关闭小地图
	this.closeMinimap = function () {
		if (!flags.__useMinimap__) {
			core.deleteCanvas("mapArrow");
			core.deleteCanvas("minimap");
			return;
		}
		var onLeft = flags.__onLeft__;
		var frame = 0;
		var x = core.__PIXELS__,
			a = 0.096,
			speed = 4.8;
		if (onLeft) {
			x = 260;
			a = -a;
			speed = -speed;
		}
		var interval = setInterval(function () {
			core.relocateCanvas("mapArrow", x - 140, 0);
			if (!onLeft)
				core.relocateCanvas("minimap", x - 120, 0);
			else core.relocateCanvas("minimap", x - 260, 0);
			speed -= a;
			x += speed;
			if (frame == 50) {
				flags.minimap = false;
				clearInterval(interval);
				core.drawClosedMap(onLeft);
				core.checkMinimap(true);
			}
			frame++;
		}, 20);
	};
	// 合上的小地图
	this.drawClosedMap = function (toLeft) {
		if (!flags.__useMinimap__) {
			core.deleteCanvas("mapArrow");
			core.deleteCanvas("minimap");
			return;
		}
		var scale = 1.3 / core.status.thisMap.width * 15 * (flags.userScale || 1);
		if (1.3 / core.status.thisMap.height * 15 * (flags.userScale || 1) < scale)
			scale = 1.3 / core.status.thisMap.height * 15 * (flags.userScale || 1);
		// 绘制
		core.createCanvas("minimap", core.__PIXELS__, 0, 120, 120, 100);
		core.createCanvas("mapArrow", core.__PIXELS__ - 20, 0, 20, 120, 100);
		core.clearMap("minimap");
		core.clearMap("mapArrow");
		if (toLeft) {
			core.relocateCanvas("minimap", -120, 0);
			core.relocateCanvas("mapArrow", 0, 0);
			document.getElementById('mapArrow').style.transform = 'rotateY(180deg)';
		}
		// 黑色底
		core.fillRect("minimap", 0, 0, 120, 120, [0, 0, 0, 0.6]);
		var config = { fromUser: true, oriFloor: core.status.floorId, scale: scale, interval: 10, noErase: true, fromMini: true };
		core.drawFlyMap("minimap", 0, 0, 120, 120, core.status.floorId, config);
		// 向左箭头
		core.fillRect("mapArrow", 0, 0, 20, 120, [230, 230, 230, 0.9]);
		core.drawLine("mapArrow", 0, 20, 20, 20, [100, 100, 100, 0.9], 2);
		core.drawLine("mapArrow", 0, 100, 20, 100, [100, 100, 100, 0.9], 2);
		core.setTextAlign("mapArrow", "center");
		core.fillText("mapArrow", "<", 10, 67, [100, 100, 100, 0.9], "20px Verdana");
		core.registerMinimapAction(true);
	};
	// 打开小地图
	this.openMinimap = function () {
		if (!flags.__useMinimap__) {
			core.deleteCanvas("mapArrow");
			core.deleteCanvas("minimap");
			return;
		}
		var onLeft = flags.__onLeft__;
		var frame = 0;
		var x = 120 + core.__PIXELS__,
			a = 0.096,
			speed = 4.8;
		if (onLeft) {
			x = 140;
			a = -a;
			speed = -speed;
		}
		var interval = setInterval(function () {
			core.relocateCanvas("mapArrow", x - 140, 0);
			if (!flags.__onLeft__)
				core.relocateCanvas("minimap", x - 120, 0);
			else core.relocateCanvas("minimap", x - 260, 0);
			speed -= a;
			x -= speed;
			if (frame == 50) {
				flags.minimap = true;
				clearInterval(interval);
				core.drawMinimap(onLeft);
				core.checkMinimap(true);
			}
			frame++;
		}, 20);
	};
	// 大地图
	this.drawTotalMap = function (floorId) {
		floorId = floorId || core.status.floorId;
		core.status.event.id = "totalMap";
		core.lockControl();
		if (!flags.viewingLayer) flags.viewingLayer = 0;
		var loop = 5;
		if (flags.worldMap) loop = core.floorIds.length;
		// 大地图时点击和键盘操作
		core.registerAction("ondown", "onDownTmap", function (x, y) {
			if (core.status.event.id == "totalMap") {
				if (y < 1 && x <= Math.floor(core.__SIZE__ / 2) - 2) { // 上移一层
					if (flags.viewingLayer < core.sortFloor().top) {
						flags.viewingLayer++;
						core.playSound('光标移动');
						core.drawTotalMap();
					}
					return true;
				}
				if (y < 1 && x >= Math.ceil(core.__SIZE__ / 2) + 1) { // 下移一层
					if (flags.viewingLayer > core.sortFloor().bottom) {
						flags.viewingLayer--;
						core.playSound('光标移动');
						core.drawTotalMap();
					}
					return true;
				}
				if (y < 1 && x < Math.ceil(core.__SIZE__ / 2) + 1 && x > Math.floor(core.__SIZE__ / 2) - 2) {
					// 区域地图
					if (flags.worldMap) {
						flags.worldMap = false;
						flags.viewingLayer = 0;
					} else flags.worldMap = true;
					core.playSound('光标移动');
					core.drawTotalMap();
					return true;
				}
				if (y >= core.__SIZE__ - 1 && x <= Math.floor(core.__SIZE__ / 2)) { // 3D
					if (core.can3D(floorId) && !flags.in3D) flags.use3D = true;
					if (flags.in3D) flags.use3D = false;
					flags.mapHint = false;
					core.playSound('光标移动');
					core.drawTotalMap();
					return true;
				}
				if (y >= core.__SIZE__ - 1 && x >= Math.ceil(core.__SIZE__ / 2)) { // hint
					if (flags.in3D) {
						if (!flags.mapHint) flags.mapHint = true;
						else flags.mapHint = false;
						core.playSound('光标移动');
						core.drawTotalMap();
					}
					return true;
				}
				flags.viewingLayer = 0;
				core.playSound("取消")
				core.deleteCanvas("mapOnUi");
				core.deleteCanvas("back");
				core.deleteCanvas("tips");
				core.closePanel();
				core.unregisterAction("ondown", "onDownTmap");
				return true;
			}
		}, 110);
		core.registerAction("keyUp", "keyUpTmap", function (keycode) {
			if (core.status.event.id == "totalMap") {
				if (keycode == 33) { // PgUp
					if (flags.viewingLayer < core.sortFloor().top) {
						flags.viewingLayer++;
						core.playSound('光标移动');
						core.drawTotalMap();
					}
					return true;
				}
				if (keycode == 34) { // PgDn
					if (flags.viewingLayer > core.sortFloor().bottom) {
						flags.viewingLayer--;
						core.playSound('光标移动');
						core.drawTotalMap();
					}
					return true;
				}
				if (keycode == 90) { // Z
					if (core.can3D(floorId) && !flags.in3D) flags.use3D = true;
					if (flags.in3D) flags.use3D = false;
					flags.mapHint = false;
					core.playSound('光标移动');
					core.drawTotalMap();
					return true;
				}
				if (keycode == 84) { // T
					if (flags.in3D) {
						if (!flags.mapHint) flags.mapHint = true;
						else flags.mapHint = false;
						core.playSound('光标移动');
						core.drawTotalMap();
					}
					return true;
				}
				if (keycode == 87) { // W
					if (flags.worldMap) {
						flags.worldMap = false;
						flags.viewingLayer = 0;
					} else flags.worldMap = true;
					core.playSound('光标移动');
					core.drawTotalMap();
					return true;
				}
				flags.viewingLayer = 0;
				core.playSound("取消")
				core.deleteCanvas("mapOnUi");
				core.deleteCanvas("back");
				core.deleteCanvas("tips");
				core.closePanel();
				core.unregisterAction("keyUp", "keyUpTmap");
				return true;
			}
		}, 110);
		// 开始画
		core.createCanvas("mapOnUi", -240, -240, core.__PIXELS__ + 480, core.__PIXELS__ + 480, 150);
		core.createCanvas("back", -240, -240, core.__PIXELS__ + 480, core.__PIXELS__ + 480, 140);
		core.createCanvas("tips", 0, 0, core.__PIXELS__, core.__PIXELS__, 160);
		var ctx = document.getElementById("tips").getContext("2d");
		ctx.shadowOffsetX = 0;
		ctx.shadowOffsetY = 0;
		ctx.shadowBlur = 5;
		ctx.shadowColor = "rgba(100, 100, 255, 1)";
		core.fillRect("back", -240, -240, core.__PIXELS__ + 480, core.__PIXELS__ + 480, [0, 0, 0, 0.9]);
		core.fillRect("tips", 0, 0, core.__PIXELS__ / 2 - 50, 32, [200, 200, 200, 0.8]);
		core.fillRect("tips", core.__PIXELS__ / 2 + 50, 0, core.__PIXELS__ / 2 - 50, 32, [200, 200, 200, 0.8]);
		core.fillRect("tips", core.__PIXELS__ / 2 - 46, 0, 92, 32, [200, 200, 200, 0.8]);
		core.drawLine("tips", 0, 32, core.__PIXELS__ / 2 - 50, 32, [50, 50, 50, 0.8], 3);
		core.drawLine("tips", core.__PIXELS__ / 2 + 50, 32, core.__PIXELS__, 32, [50, 50, 50, 0.8], 3);
		core.drawLine("tips", core.__PIXELS__ / 2 - 46, 32, core.__PIXELS__ / 2 + 46, 32, [50, 50, 50, 0.8], 3);
		core.fillRect("tips", 0, core.__PIXELS__, core.__PIXELS__ / 2 - 5, -32, [200, 200, 200, 0.8]);
		core.fillRect("tips", core.__PIXELS__ / 2 + 5, core.__PIXELS__, core.__PIXELS__ / 2 - 5, -32, [200, 200, 200, 0.8]);
		core.drawLine("tips", 0, core.__PIXELS__ - 32, core.__PIXELS__ / 2 - 5, core.__PIXELS__ - 32, [50, 50, 50, 0.8], 3);
		core.drawLine("tips", core.__PIXELS__ / 2 + 5, core.__PIXELS__ - 32, core.__PIXELS__, core.__PIXELS__ - 32, [50, 50, 50, 0.8], 3);
		core.setTextAlign("tips", "center");
		core.fillText("tips", "上移一层", core.__PIXELS__ / 4 - 23, 24, [255, 255, 255, 0.8], "24px " + core.status.globalAttribute.font);
		core.fillText("tips", "下移一层", core.__PIXELS__ / 4 * 3 + 23, 24, [255, 255, 255, 0.8], "24px " + core.status.globalAttribute.font);
		core.fillText("tips", flags.worldMap ? "小地图" : "区域地图", core.__PIXELS__ / 2, 24, [255, 255, 255, 0.8], "24px " + core.status.globalAttribute.font);
		core.drawFlyMap("mapOnUi", 240, 240, core.__PIXELS__, core.__PIXELS__,
			floorId, { fromUser: true, opacity: 1, oriFloor: floorId, noErase: true, use3D: flags.use3D, layer: flags.viewingLayer, loop: loop, clearCache: true });
		if (flags.in3D)
			core.fillText("tips", "参考线（T）", core.__PIXELS__ / 4 * 3, core.__PIXELS__ - 8, [255, 255, 255, 0.8], "24px " + core.status.globalAttribute.font);
		if (core.can3D(floorId) && !flags.in3D)
			core.fillText("tips", "3D模式（Z）", core.__PIXELS__ / 4, core.__PIXELS__ - 8, [255, 255, 255, 0.8], "24px " + core.status.globalAttribute.font);
		if (flags.in3D)
			core.fillText("tips", "2D模式（Z）", core.__PIXELS__ / 4, core.__PIXELS__ - 8, [255, 255, 255, 0.8], "24px " + core.status.globalAttribute.font);
		if (flags.mapHint) {
			core.drawLine("back", 240, 240 + core.__PIXELS__, 240 + core.__PIXELS__, 240, [100, 100, 240, 0.4], 2);
			core.drawLine("back", 240 + core.__PIXELS__ / 2, 240, 240 + core.__PIXELS__ / 2, 240 + core.__PIXELS__, [100, 100, 240, 0.4], 2);
			core.drawLine("back", 240, 240 + core.__PIXELS__ / 2, 240 + core.__PIXELS__, 240 + core.__PIXELS__ / 2, [100, 100, 240, 0.4], 2);
		}
	};
},
    "宝石血瓶显示": function () {
	/* 宝石血瓶左下角显示数值
	 * 需要将 变量：itemDetail改为true才可正常运行
	 * 请尽量减少勇士的属性数量，否则可能会出现严重卡顿
	 * 注意：这里的属性必须是core.status.hero里面的，flag无法显示
	 * 如果不想显示，可以core.setFlag("itemDetail", false);
	 * 然后再core.getItemDetail();
	 * 如有bug在大群或造塔群@古祠
	 */

	// 谁tm在即捡即用效果里面调用带有含刷新状态栏的函数
	var origin = core.control.updateStatusBar;
	core.updateStatusBar = core.control.updateStatusBar = function () {
		if (core.getFlag('__statistics__')) return;
		else return origin.apply(core.control, arguments);
	}

	core.bigmap.threshold = 256;

	core.control.updateDamage = function (floorId, ctx) {
		floorId = floorId || core.status.floorId;
		if (!floorId || core.status.gameOver || main.mode != 'play') return;
		var onMap = ctx == null;

		// 没有怪物手册
		if (!core.hasItem('book')) return;
		core.status.damage.posX = core.bigmap.posX;
		core.status.damage.posY = core.bigmap.posY;
		if (!onMap) {
			var width = core.floors[floorId].width,
				height = core.floors[floorId].height;
			// 地图过大的缩略图不绘制显伤
			if (width * height > core.bigmap.threshold) return;
		}
		this._updateDamage_damage(floorId, onMap);
		this._updateDamage_extraDamage(floorId, onMap);
		core.getItemDetail(floorId); // 宝石血瓶详细信息
		this.drawDamage(ctx);
	};
	// 绘制地图显示
	control.prototype._drawDamage_draw = function (ctx, onMap) {
		if (!core.hasItem('book')) return;
		// *** 下一句话可以更改你想要的显示字体
		core.setFont(ctx, "bold 11px Arial");
		// ***
		core.setTextAlign(ctx, 'left');
		core.status.damage.data.forEach(function (one) {
			var px = one.px,
				py = one.py;
			if (onMap && core.bigmap.v2) {
				px -= core.bigmap.posX * 32;
				py -= core.bigmap.posY * 32;
				if (px < -32 * 2 || px > core.__PX__ + 32 || py < -32 || py > core.__PY__ + 32)
					return;
			}
			core.fillBoldText(ctx, one.text, px, py, one.color);
		});
		core.setTextAlign(ctx, 'center');
		core.status.damage.extraData.forEach(function (one) {
			var px = one.px,
				py = one.py;
			if (onMap && core.bigmap.v2) {
				px -= core.bigmap.posX * 32;
				py -= core.bigmap.posY * 32;
				if (px < -32 || px > core.__PX__ + 32 || py < -32 || py > core.__PY__ + 32)
					return;
			}
			core.fillBoldText(ctx, one.text, px, py, one.color);
		});
	};
	// 获取宝石信息 并绘制
	this.getItemDetail = function (floorId) {
		if (!core.getFlag("itemDetail")) return;
		floorId = floorId || core.status.thisMap.floorId;
		core.status.maps[floorId].blocks.forEach(function (block) {
			if (block.event.cls !== 'items' || block.event.id === 'superPotion') return;
			var x = block.x,
				y = block.y;
			// v2优化，只绘制范围内的部分
			if (core.bigmap.v2) {
				if (x < core.bigmap.posX - core.bigmap.extend || x > core.bigmap.posX + core.__SIZE__ + core.bigmap.extend ||
					y < core.bigmap.posY - core.bigmap.extend || y > core.bigmap.posY + core.__SIZE__ + core.bigmap.extend) {
					return;
				}
			}
			var id = block.event.id;
			var item = core.material.items[id];
			if (item.cls === 'equips') {
				// 装备也显示
				var diff = core.clone(item.equip.value || {});
				var per = item.equip.percentage;
				for (var name in per) {
					diff[name + 'per'] = per[name].toString() + '%';
				}
				drawItemDetail(diff, x, y);
				return;
			}
			var before = core.clone(core.status.hero);
			// 跟数据统计原理一样 执行效果 前后比较
			core.setFlag("__statistics__", true);
			try {
				eval(item.itemEffect);
			} catch (error) {}
			var diff = compareObject(before, core.status.hero);
			core.status.hero = hero = before;
			flags = core.status.hero.flags;
			drawItemDetail(diff, x, y);
		});
	};
	// 比较两个对象之间每一项的数值差异（弱等于） 返回数值差异
	function compareObject(a, b) {
		a = a || {};
		b = b || {};
		var diff = {}; // 差异
		for (var name in a) {
			diff[name] = b[name] - (a[name] || 0);
			if (!diff[name]) diff[name] = void 0;
		}
		return diff;
	};
	// 绘制
	function drawItemDetail(diff, x, y) {
		var px = 32 * x + 2,
			py = 32 * y + 30;
		var content = "";
		// 获得数据和颜色
		var i = 0;
		for (var name in diff) {
			if (!diff[name]) continue;
			var color = "#ffffff";
			if (typeof diff[name] === 'number')
				diff[name] = core.formatBigNumber(diff[name], true);
			switch (name) {
			case 'atk':
			case 'atkper':
				color = "#FF7A7A";
				break;
			case 'def':
			case 'defper':
				color = "#00E6F1";
				break;
			case 'mdef':
			case 'mdefper':
				color = "#6EFF83";
				break;
			case 'hp':
				color = "#A4FF00";
				break;
			case 'hpmax':
			case 'hpmaxper':
				color = "#F9FF00";
				break;
			case 'mana':
				color = "#cc6666";
				break;
			}
			content = diff[name];
			// 绘制
			core.status.damage.data.push({ text: content, px: px, py: py - 10 * i, color: color });
			i++;
		}
	}
},
    "自动清零伤怪": function () {
	/**
	 * --------------- 安装说明 ---------------
	 *
	 * 复制到插件编写里面即可
	 *
	 * --------------- 使用说明 ---------------
	 *
	 * 把变量 __autoBattle__ 设置成true开启，设置成false关闭，然后在想要清的时刻，比如战后，每步后等加上core.autoBattle()即可
	 *
	 * --------------- 自定义说明 ---------------
	 *
	 * 可以更改canBattle函数自定义某个怪物是否应该被清
	 */

	/**
	 * 是否清这个怪，可以修改这里来实现对不同怪的不同操作
	 * @param {string} enemy
	 * @param {number} x
	 * @param {number} y
	 */
	function canBattle(enemy, x, y) {
		const loc = `${x},${y}`;
		const floor = core.floors[core.status.floorId];
		const e = core.material.enemys[enemy];
		const hasEvent =
			has(floor.afterBattle[loc]) ||
			has(floor.beforeBattle[loc]) ||
			has(e.beforeBattle) ||
			has(e.afterBattle) ||
			has(floor.events[loc]);
		// 有事件，不清
		if (hasEvent) return false;
		const damage = core.getDamageInfo(enemy, void 0, x, y)?.damage;
		// 伤害低于1，清
		if (has(damage) && damage <= core.getFlag("lessthan", 1)) return true;
		return false;
	}

	/**
	 * @template T
	 * @param {T} v
	 * @returns {v is NonNullable<T>}
	 */
	function has(v) {
		return v !== null && v !== undefined;
	}

	/**
	 * 广搜，搜索可以到达的需要清的怪
	 * @param {string} floorId
	 */
	function bfs(floorId, deep = Infinity) {
		core.extractBlocks(floorId);
		const objs = core.getMapBlocksObj(floorId);
		const { x, y } = core.status.hero.loc;
		/** @type {[direction, number, number][]} */
		const dir = Object.entries(core.utils.scan).map(v => [
			v[0],
			v[1].x,
			v[1].y
		]);

		/** @type {[number, number][]} */
		const queue = [
			[x, y]
		];
		const mapped = {
			[`${x},${y}`]: true
		};
		while (queue.length > 0 && deep > 0) {
			const [nx, ny] = queue.shift();
			dir.forEach(v => {
				const [tx, ty] = [nx + v[1], ny + v[2]];
				const loc = `${tx},${ty}`;
				if (mapped[loc]) return;
				const block = objs[loc];
				mapped[loc] = true;
				const isEnemy =
					has(block) && block.event.cls.startsWith('enemy');
				const changeFloor = core.floors[floorId].changeFloor[loc];
				if (
					(!core.canMoveHero(nx, ny, v[0]) && !isEnemy) ||
					block?.event.cls === 'items' ||
					(core.noPass(tx, ty) && !isEnemy) ||
					has(core.floors[floorId].events[loc]) ||
					(has(changeFloor) &&
						changeFloor.ignoreChangeFloor === false)
				)
					return;

				if (isEnemy) {
					if (canBattle(block.event.id, tx, ty) && !block.disable) {
						// 能打且没有事件就直接干他丫的
						core.battle(block.event.id, tx, ty);
					} else {
						return;
					}
				}
				// 然后判断目标点是否有地图伤害等，没有就直接添加到队列
				const damage = core.status.checkBlock.damage[loc];
				const ambush = core.status.checkBlock.ambush[loc];
				const repulse = core.status.checkBlock.repulse[loc];
				if (
					(!has(damage) || (has(damage) && damage <= 0)) &&
					!has(ambush) &&
					!has(repulse)
				)
					queue.push([tx, ty]);
			});
			deep--;
		}
	}

	this.autoBattle = function () {
		if (!flags.__autoBattle__) return;
		// 如果勇士当前点有地图伤害，只清周围，如果有时间，直接不清了
		const { x, y } = core.status.hero.loc;
		const floor = core.floors[core.status.floorId];
		const hasEvent = has(floor.events[`${x},${y}`]);
		if (hasEvent) return;
		const damage = core.status.checkBlock.damage[`${x},${y}`];
		let deep = Infinity;
		if (has(damage) && damage > 0) {
			deep = 1;
		}
		bfs(core.status.floorId, deep);
	};
},
    "清理难以减伤怪物": function () {
	/**
	 * --------------- 安装说明 ---------------
	 *
	 * 复制到插件编写里面即可
	 *
	 * --------------- 使用说明 ---------------
	 *
	 * 把变量 __autoBattle__ 设置成true开启，设置成false关闭，然后在想要清的时刻，比如战后，每步后等加上core.autoBattle()即可
	 *
	 * --------------- 自定义说明 ---------------
	 *
	 * 可以更改canBattle函数自定义某个怪物是否应该被清
	 */

	/**
	 * 是否清这个怪，可以修改这里来实现对不同怪的不同操作
	 * @param {string} enemy
	 * @param {number} x
	 * @param {number} y
	 */
	function canBattle(enemy, x, y) {
		const loc = `${x},${y}`;
		const floor = core.floors[core.status.floorId];
		const e = core.material.enemys[enemy];
		const hasEvent =
			has(floor.afterBattle[loc]) ||
			has(floor.beforeBattle[loc]) ||
			has(e.beforeBattle) ||
			has(e.afterBattle) ||
			has(floor.events[loc]);
		// 有事件，不清
		if (hasEvent) return false;
		const damage = core.getDamageInfo(enemy, void 0, x, y)?.damage;
		// 魔防不减伤，清
		if (has(damage) && ((core.getDamage(core.getBlockId(x, y), x, y) === core.getDamageInfo(core.getBlockId(x, y), { 'atk': hero.atk, 'def': hero.def, 'mdef': (hero.mdef + 10000000) }, x, y).damage))) return true;
		return false;
	}

	/**
	 * @template T
	 * @param {T} v
	 * @returns {v is NonNullable<T>}
	 */
	function has(v) {
		return v !== null && v !== undefined;
	}

	/**
	 * 广搜，搜索可以到达的需要清的怪
	 * @param {string} floorId
	 */
	function bfs(floorId, deep = Infinity) {
		core.extractBlocks(floorId);
		const objs = core.getMapBlocksObj(floorId);
		const { x, y } = core.status.hero.loc;
		/** @type {[direction, number, number][]} */
		const dir = Object.entries(core.utils.scan).map(v => [
			v[0],
			v[1].x,
			v[1].y
		]);

		/** @type {[number, number][]} */
		const queue = [
			[x, y]
		];
		const mapped = {
			[`${x},${y}`]: true
		};
		while (queue.length > 0 && deep > 0) {
			const [nx, ny] = queue.shift();
			dir.forEach(v => {
				const [tx, ty] = [nx + v[1], ny + v[2]];
				const loc = `${tx},${ty}`;
				if (mapped[loc]) return;
				const block = objs[loc];
				mapped[loc] = true;
				const isEnemy =
					has(block) && block.event.cls.startsWith('enemy');
				const changeFloor = core.floors[floorId].changeFloor[loc];
				if (
					(!core.canMoveHero(nx, ny, v[0]) && !isEnemy) ||
					block?.event.cls === 'items' ||
					(core.noPass(tx, ty) && !isEnemy) ||
					has(core.floors[floorId].events[loc]) ||
					(has(changeFloor) &&
						changeFloor.ignoreChangeFloor === false)
				)
					return;

				if (isEnemy) {
					if (canBattle(block.event.id, tx, ty) && !block.disable) {
						// 能打且没有事件就直接干他丫的
						core.battle(block.event.id, tx, ty);
					} else {
						return;
					}
				}
				// 然后判断目标点是否有地图伤害等，没有就直接添加到队列
				const damage = core.status.checkBlock.damage[loc];
				const ambush = core.status.checkBlock.ambush[loc];
				const repulse = core.status.checkBlock.repulse[loc];
				if (
					(!has(damage) || (has(damage) && damage <= 0)) &&
					!has(ambush) &&
					!has(repulse)
				)
					queue.push([tx, ty]);
			});
			deep--;
		}
	}

	this.autoDefeat = function () {
		if (!flags.__autoBattle__) return;
		// 如果勇士当前点有地图伤害，只清周围，如果有时间，直接不清了
		const { x, y } = core.status.hero.loc;
		const floor = core.floors[core.status.floorId];
		const hasEvent = has(floor.events[`${x},${y}`]);
		if (hasEvent) return;
		const damage = core.status.checkBlock.damage[`${x},${y}`];
		let deep = Infinity;
		if (has(damage) && damage > 0) {
			deep = 1;
		}
		bfs(core.status.floorId, deep);
	};
}
}